package st.tori.jpredict.core.util;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

public class DateUtil {

	private static Logger logger = Logger.getLogger(DateUtil.class.getName());
	public static int SERVER_OFFSET_FROM_UTC_IN_HOUR		 = +9;
	
	/*
	 * 今日の日付を返すメソッド
	 */
	public static String getTodayStr(){
		return getDateStr(new Date(),"yyyy/MM/dd");
	}
	/*
	 * 現在時刻を返すメソッド
	 */
	public static String getNowStr(){
		return getDateStr(new Date(),"yyyy/MM/dd HH:mm");
	}
	/*
	 * 時刻を指定されたフォーマットで返すメソッド
	 */
	public static String getDateStr(Date date,String format){
		if(date==null)return null;
		return new SimpleDateFormat(format).format(date);
	}
	public static String getDateStr(Calendar cal,String format){
		if(cal==null)return null;
		return getDateStr(cal.getTime(),format);
	}
	public static String getDateStr(Date date,DateFormat format){
		if(date==null||format==null)return null;
		return format.format(date);
	}

	private static final String DEFAULT_DATE_STR_FORMAT = "yyyy-MM-dd HH:mm:ss";
	public static String getDateStr(){	return getDateStr(new Date(),DEFAULT_DATE_STR_FORMAT);	}
	public static String getDateStr(long timeInMillis){
		Calendar cal = new GregorianCalendar();
		cal.setTimeInMillis(timeInMillis);
		return getDateStr(cal,DEFAULT_DATE_STR_FORMAT);
	}
	public static String getDateStr(Date date){	return getDateStr(date,DEFAULT_DATE_STR_FORMAT);	}
	public static String getDateStr(Calendar cal){
		if(cal==null)return null;
		return getDateStr(cal.getTime(),DEFAULT_DATE_STR_FORMAT);
	}
	/*
	 * 指定されたフォーマットの文字列をDateで返すメソッド
	 */
	public static Date getStrDate(String str,String format){
		if(str==null||format==null)return null;
		SimpleDateFormat sdf = new SimpleDateFormat();
		sdf.setLenient( true );
		sdf.applyPattern(format);
		
		Date date = null;
		try {
			date = sdf.parse(str);
		} catch (ParseException e) {
			//e.printStackTrace();
			date = null;
		}
		return date;
	}
	public static Date getStrDate(String str){
		return getStrDate(str,DEFAULT_DATE_STR_FORMAT);
	}
	public static Date getLongDate(String str){
		if(str==null)return null;
		long val = StringUtil.forceLong(str, -1);
		if(val<=0)return null;
		return new Date(val);
	}
	
	public static final int ASTRO_CAPRICORN		 = 101; //	やぎ座
	public static final int ASTRO_AQUARIUS		 = 102; //	みずがめ座
	public static final int ASTRO_PISCES		 = 103; //	うお座
	public static final int ASTRO_ARIES			 = 104; //	おひつじ座
	public static final int ASTRO_TAURUS		 = 105; //	おうし座
	public static final int ASTRO_GEMINI		 = 106; //	ふたご座
	public static final int ASTRO_CANCER		 = 107; //	かに座
	public static final int ASTRO_LEO			 = 108; //	しし座
	public static final int ASTRO_VIRGO			 = 109; //	おとめ座
	public static final int ASTRO_LIBRA			 = 110; //	てんびん座
	public static final int ASTRO_SCORPIO		 = 111; //	さそり座
	public static final int ASTRO_SAGITTARIUS	 = 112; //	いて座
	public static Map ASTRO_KANA_MAP = new HashMap();
	public static Map ASTRO_KANJI_MAP = new HashMap();
	static{
		ASTRO_KANA_MAP.put(Integer.toString(ASTRO_CAPRICORN),"やぎ座");
		ASTRO_KANA_MAP.put(Integer.toString(ASTRO_AQUARIUS),"みずがめ座");
		ASTRO_KANA_MAP.put(Integer.toString(ASTRO_PISCES),"うお座");
		ASTRO_KANA_MAP.put(Integer.toString(ASTRO_ARIES),"おひつじ座");
		ASTRO_KANA_MAP.put(Integer.toString(ASTRO_TAURUS),"おうし座");
		ASTRO_KANA_MAP.put(Integer.toString(ASTRO_GEMINI),"ふたご座");
		ASTRO_KANA_MAP.put(Integer.toString(ASTRO_CANCER),"かに座");
		ASTRO_KANA_MAP.put(Integer.toString(ASTRO_LEO),"しし座");
		ASTRO_KANA_MAP.put(Integer.toString(ASTRO_VIRGO),"おとめ座");
		ASTRO_KANA_MAP.put(Integer.toString(ASTRO_LIBRA),"てんびん座");
		ASTRO_KANA_MAP.put(Integer.toString(ASTRO_SCORPIO),"さそり座");
		ASTRO_KANA_MAP.put(Integer.toString(ASTRO_SAGITTARIUS),"いて座");
		
		ASTRO_KANJI_MAP.put(Integer.toString(ASTRO_CAPRICORN),"山羊座");
		ASTRO_KANJI_MAP.put(Integer.toString(ASTRO_AQUARIUS),"水瓶座");
		ASTRO_KANJI_MAP.put(Integer.toString(ASTRO_PISCES),"魚座");
		ASTRO_KANJI_MAP.put(Integer.toString(ASTRO_ARIES),"牡羊座");
		ASTRO_KANJI_MAP.put(Integer.toString(ASTRO_TAURUS),"牡牛座");
		ASTRO_KANJI_MAP.put(Integer.toString(ASTRO_GEMINI),"双子座");
		ASTRO_KANJI_MAP.put(Integer.toString(ASTRO_CANCER),"蟹座");
		ASTRO_KANJI_MAP.put(Integer.toString(ASTRO_LEO),"獅子座");
		ASTRO_KANJI_MAP.put(Integer.toString(ASTRO_VIRGO),"乙女座");
		ASTRO_KANJI_MAP.put(Integer.toString(ASTRO_LIBRA),"天秤座");
		ASTRO_KANJI_MAP.put(Integer.toString(ASTRO_SCORPIO),"蠍座");
		ASTRO_KANJI_MAP.put(Integer.toString(ASTRO_SAGITTARIUS),"射手座");
	}
	
	/**
	 * 日付から星座を求める
	 *
	 * @param date
	 */
	public static final int getAstroByDate(Date date) {
		GregorianCalendar cal = new GregorianCalendar();
		cal.setTime(date);
		int yday = cal.get(Calendar.DAY_OF_YEAR);
		//閏年で3月以降の日付なら日数を-1する
		if(cal.isLeapYear(cal.get(Calendar.YEAR)) && cal.get(Calendar.MONTH) >= 2) yday--;
		if(yday < 20) return ASTRO_CAPRICORN;
		else if(yday < 50) return ASTRO_AQUARIUS;
		else if(yday < 79) return ASTRO_PISCES;
		else if(yday < 110) return ASTRO_ARIES;
		else if(yday < 141) return ASTRO_TAURUS;
		else if(yday < 173) return ASTRO_GEMINI;
		else if(yday < 204) return ASTRO_CANCER;
		else if(yday < 235) return ASTRO_LEO;
		else if(yday < 266) return ASTRO_VIRGO;
		else if(yday < 296) return ASTRO_LIBRA;
		else if(yday < 325) return ASTRO_SCORPIO;
		else if(yday < 355) return ASTRO_SAGITTARIUS;
		else return ASTRO_CAPRICORN;
	}

	public static final int getDaysFrom(Date date){
		return getDaysBetween(new Date(),date);
	}
	public static final int getDaysBetween(Date date1,Date date2){
		if(date1==null||date2==null)return -1;
		Calendar cal1 = new GregorianCalendar();
		Calendar cal2 = new GregorianCalendar();
		cal1.setTime(date1);
		cal2.setTime(date2);
		return getDaysBetween(cal1,cal2);
	}
	public static final int getDaysFrom(Calendar cal){
		return getDaysBetween(new GregorianCalendar(),cal);
	}
	public static final int getDaysBetween(Calendar cal1,Calendar cal2){
		int year1 = cal1.get(Calendar.YEAR);
		int month1 = cal1.get(Calendar.MONTH);
		int day1 = cal1.get(Calendar.DATE);
		int year2 = cal2.get(Calendar.YEAR);
		int month2 = cal2.get(Calendar.MONTH);
		int day2 = cal2.get(Calendar.DATE);
		Calendar _cal1 = new GregorianCalendar(year1,month1,day1);
		Calendar _cal2 = new GregorianCalendar(year2,month2,day2);
		long time1 = _cal1.getTimeInMillis();
		long time2 = _cal2.getTimeInMillis();
		return (int)(Math.abs(time1-time2)/(24*60*60*1000));
	}

	public static final Date getTodayAsDate(){
		Calendar cal = new GregorianCalendar();
		return parseCal2Date(cal);
	}
	public static final Date getWeekFirstDayAsDate(int dow){
		Calendar cal = new GregorianCalendar();
		while(cal.get(Calendar.DAY_OF_WEEK)!=dow)cal.add(Calendar.DATE, -1);
		return parseCal2Date(cal);
	}
	public static final Date getMonthFirstDayAsDate(){
		Calendar cal = new GregorianCalendar();
		cal.set(Calendar.DATE,1);
		return parseCal2Date(cal);
	}
	private static Date parseCal2Date(Calendar cal){
		if(cal==null)return null;
		cal = new GregorianCalendar(cal.get(Calendar.YEAR),cal.get(Calendar.MONTH),cal.get(Calendar.DATE));
		return cal.getTime();
	}
	
	private static final String DOW_SUNDAY		 = "sun";	//Util.getMessageResource("js.dict.sun");
	private static final String DOW_MONDAY		 = "mon";	//Util.getMessageResource("js.dict.mon");
	private static final String DOW_TUESDAY		 = "tue";	//Util.getMessageResource("js.dict.tue");
	private static final String DOW_WEDNESDAY	 = "wed";	//Util.getMessageResource("js.dict.wed");
	private static final String DOW_THURSDAY	 = "thu";	//Util.getMessageResource("js.dict.thu");
	private static final String DOW_FRIDAY		 = "fri";	//Util.getMessageResource("js.dict.fri");
	private static final String DOW_SATURDAY	 = "sat";	//Util.getMessageResource("js.dict.sat");
	private static final String DOW_YOU			 = "you";	//Util.getMessageResource("js.dict.you");
	private static final String DOW_YOUBI		 = "youbi";	//Util.getMessageResource("js.dict.youbi");
	/*
	 * Calendar.SUNDAY等を「日曜」「月曜」「火曜」･･･という文字列に変換するメソッド
	 */
	public static final String getDowStrWithYou(Date date){
		String str = getDow(date);
		return (str==null)?null:str+DOW_YOU;
	}
	/*
	 * Calendar.SUNDAY等を「日曜日」「月曜日」「火曜日」･･･という文字列に変換するメソッド
	 */
	public static final String getDowStrWithYoubi(Date date){
		String str = getDow(date);
		return (str==null)?null:str+DOW_YOUBI;
	}
	/*
	 * Calendar.SUNDAY等を「日」「月」「火」･･･という文字列に変換するメソッド
	 */
	public static final String getDowStr(Date date){
		return getDow(date);
	}
	private static final int getDowInt(Date date){
		if(date==null)return -1;
		Calendar cal = new GregorianCalendar();
		cal.setTime(date);
		return cal.get(Calendar.DAY_OF_WEEK);
	}
	private static final String getDow(Date date){
		int dow = getDowInt(date);
		if(Calendar.SUNDAY==dow)return DOW_SUNDAY;
		if(Calendar.MONDAY==dow)return DOW_MONDAY;
		if(Calendar.TUESDAY==dow)return DOW_TUESDAY;
		if(Calendar.WEDNESDAY==dow)return DOW_WEDNESDAY;
		if(Calendar.THURSDAY==dow)return DOW_THURSDAY;
		if(Calendar.FRIDAY==dow)return DOW_FRIDAY;
		if(Calendar.SATURDAY==dow)return DOW_SATURDAY;
		return null;
	}
	public static final boolean isWeekend(Date date){
		int dow = getDowInt(date);
		return (Calendar.SUNDAY==dow||Calendar.SATURDAY==dow);
	}

	private static final Calendar GANNEN_HEISEI	 = new GregorianCalendar(1989,1-1,8);
	private static final Calendar GANNEN_SHOWA	 = new GregorianCalendar(1926,12-1,25);
	private static final Calendar GANNEN_TAISYO	 = new GregorianCalendar(1912,7-1,30);
	private static final Calendar GANNEN_MEIJI	 = new GregorianCalendar(1868,1-1,25);
	/*
	 * 明治以降の年号を返すメソッド
	 */
	public static String getNengo(Date date) {
		if(date==null)return null;
		Calendar cal = new GregorianCalendar();
		cal.setTime(date);
		return getNengo(cal);
	}
	/*
	 * 明治以降の年号を返すメソッド
	 */
	public static String getNengo(Calendar cal) {
		if(cal==null)return null;
		int year = cal.get(Calendar.YEAR);
		Calendar tmpCal = new GregorianCalendar(year,cal.get(Calendar.MONTH),cal.get(Calendar.DATE));
		if(tmpCal.after(GANNEN_HEISEI))
			return (year==1989)?"平成元年":"平成"+(year-1989+1)+"年";
		else if(tmpCal.after(GANNEN_SHOWA))
			return (year==1926)?"昭和元年":"昭和"+(year-1926+1)+"年";
		else if(tmpCal.after(GANNEN_TAISYO))
			return (year==1912)?"大正元年":"大正"+(year-1912+1)+"年";
		else if(tmpCal.after(GANNEN_MEIJI))
			return (year==1868)?"明治元年":"明治"+(year-1868+1)+"年";
		else
			return null;
	}
	/*
	 * 2つの日付の間の日数を返すメソッド（sinceCal<=untilCalでなければ0を返す）
	 */
	public static int getDayCount(Date sinceDate,Date untilDate){
		if(sinceDate==null||untilDate==null)return 0;
		Calendar sinceCal = new GregorianCalendar();sinceCal.setTime(sinceDate);
		Calendar untilCal = new GregorianCalendar();untilCal.setTime(untilDate);
		return getDayCount(sinceCal,untilCal);
	}
	public static int getDayCount(Calendar sinceCal,Calendar untilCal){
		if(sinceCal==null||untilCal==null)return 0;
		Calendar tmpSinceCal = new GregorianCalendar(sinceCal.get(Calendar.YEAR),sinceCal.get(Calendar.MONTH),sinceCal.get(Calendar.DATE));
		Calendar tmpUntilCal = new GregorianCalendar(untilCal.get(Calendar.YEAR),untilCal.get(Calendar.MONTH),untilCal.get(Calendar.DATE));
		if(!tmpSinceCal.before(tmpUntilCal))return 0;
		long diff = tmpUntilCal.getTimeInMillis() - tmpSinceCal.getTimeInMillis();
		double dayCount = diff/(1000*60*60*24);
		//bnd.info("DateUtil.getDayCount("+StringUtil.formatDate(sinceCal.getTime(), "yyyy-MM-dd HH:mm:ss")+","+StringUtil.formatDate(untilCal.getTime(), "yyyy-MM-dd HH:mm:ss")+")="+((int)dayCount));
		return (int)dayCount;
	}
	public static final int getWeekCount(Date sinceDate,Date untilDate) {
		if(sinceDate==null||untilDate==null)return 0;
		Calendar sinceCal = new GregorianCalendar();sinceCal.setTime(sinceDate);
		Calendar untilCal = new GregorianCalendar();untilCal.setTime(untilDate);
		return getWeekCount(sinceCal,untilCal);
	}
	public static final int getWeekCount(Calendar sinceCal,Calendar untilCal){
		if(sinceCal==null||untilCal==null)return 0;
		int dayCount = getDayCount(sinceCal,untilCal);
		if(dayCount<0)return 0;
		int weekCount = dayCount/7;
		if(dayCount%7>0)weekCount++;
		//bnd.info("DateUtil.getWeekCount("+StringUtil.formatDate(sinceCal.getTime(), "yyyy-MM-dd HH:mm:ss")+","+StringUtil.formatDate(untilCal.getTime(), "yyyy-MM-dd HH:mm:ss")+")="+weekCount);
		return weekCount;
	}

	public static boolean isDoNichiShuku(Calendar cal){
		if(cal==null)return false;
		int dow = cal.get(Calendar.DAY_OF_WEEK);
		return (dow==Calendar.SATURDAY||dow==Calendar.SUNDAY||isHolidayNational(cal, false));
	}
	public static boolean isHolidayNational(Calendar cal){
		return isHolidayNational(cal, false);
	}
	public static boolean isHolidayNational(Calendar cal, boolean ignoreHurikae){
		if(cal==null)return false;
		return (isHolidayGanjitsu(cal)
			||isHolidaySeijin(cal)
			||isHolidayKenkoku(cal)
			||isHolidayShowa(cal)
			||isHolidayKenpo(cal)
			||isHolidayMidori(cal)
			||isHolidayKodomo(cal)
			||isHolidayUmi(cal)
			||isHolidayKeiro(cal)
			||isHolidayShunbun(cal)
			||isHolidayTaiiku(cal)
			||isHolidayBunka(cal)
			||isHolidayKinro(cal)
			||isHolidayTenno(cal)
			||(!ignoreHurikae && isHolidayHurikae(cal))
			||(!ignoreHurikae && isHolidayKokumin(cal)));
	}
	public static boolean isHolidayGanjitsu(Calendar cal){	return isSpecificDate(cal, 1, 1);	}
	public static boolean isHolidaySeijin(Calendar cal){	return isHolidayDow(cal,1,Calendar.MONDAY,2);	}
	public static boolean isHolidayKenkoku(Calendar cal){
		if(cal==null)return false;
		int year = cal.get(Calendar.YEAR);
		if(year%4==0){
			if(year<=1956)
				return isSpecificDate(cal,3,21);
			if(year<=2088)
				return isSpecificDate(cal,3,20);
			if(year<=2096)
				return isSpecificDate(cal,3,19);
		}else if(year%4==1){
			if(year<=1989)
				return isSpecificDate(cal,3,21);
			if(year<=2098)
				return isSpecificDate(cal,3,20);
		}else if(year%4==2){
			if(year<=2022)
				return isSpecificDate(cal,3,21);
			if(year<=2098)
				return isSpecificDate(cal,3,20);
		}else if(year%4==3){
			if(year<=1923)
				return isSpecificDate(cal,3,22);
			if(year<=2099)
				return isSpecificDate(cal,3,20);
		}
		return false;
	}
	public static boolean isHolidayShowa(Calendar cal){	return isSpecificDate(cal, 4, 29);	}
	public static boolean isHolidayKenpo(Calendar cal){	return isSpecificDate(cal, 5, 3);	}
	public static boolean isHolidayMidori(Calendar cal){	return isSpecificDate(cal, 5, 4);	}
	public static boolean isHolidayKodomo(Calendar cal){	return isSpecificDate(cal, 5, 5);	}
	public static boolean isHolidayUmi(Calendar cal){	return isHolidayDow(cal,7,Calendar.MONDAY,3);	}
	public static boolean isHolidayKeiro(Calendar cal){	return isHolidayDow(cal,9,Calendar.MONDAY,3);	}
	public static boolean isHolidayShunbun(Calendar cal){
		if(cal==null)return false;
		int year = cal.get(Calendar.YEAR);
		if(year<=2011)
			return isSpecificDate(cal, 9, 23);
		if(year<=2044){
			if(isUruuYear(year))
				return isSpecificDate(cal, 9, 23);
			else
				return isSpecificDate(cal, 9, 22);
		}
		return false;
	}
	public static boolean isToji(Calendar cal){
		if(cal==null)return false;
		int year = cal.get(Calendar.YEAR);
		if(year%4==0){
			return (year<=1988)?isSpecificDate(cal, 12, 22):isSpecificDate(cal, 12, 21);
		}else if(year%4==1){
			return (year<=2025)?isSpecificDate(cal, 12, 22):isSpecificDate(cal, 12, 21);
		}else if(year%4==2){
			if(year<=1918)
				return isSpecificDate(cal, 12, 23);
			else if(year<=2058)
				return isSpecificDate(cal, 12, 22);
			else
				return isSpecificDate(cal, 12, 21);
		}else{
			if(year<=1955)
				return isSpecificDate(cal, 12, 23);
			else if(year<=2095)
				return isSpecificDate(cal, 12, 22);
			else
				return isSpecificDate(cal, 12, 21);
		}
	}
	public static boolean isHolidayTaiiku(Calendar cal){	return isHolidayDow(cal,10,Calendar.MONDAY,2);	}
	public static boolean isHolidayBunka(Calendar cal){	return isSpecificDate(cal, 11, 3);	}
	public static boolean isHolidayKinro(Calendar cal){	return isSpecificDate(cal, 11, 23);	}
	public static boolean isHolidayTenno(Calendar cal){	return isSpecificDate(cal, 12, 23);	}
	public static boolean isHolidayHurikae(Calendar cal){
		if(cal==null)return false;
		Calendar yesterday = getYesterday(cal);
		return isHolidayNational(yesterday, true)&&yesterday.get(Calendar.DAY_OF_WEEK)==Calendar.SUNDAY;
	}
	public static boolean isHolidayKokumin(Calendar cal){
		if(cal==null)return false;
		if(cal.get(Calendar.DAY_OF_WEEK)==Calendar.SUNDAY)return false;
		Calendar yesterday = getYesterday(cal);
		Calendar tomorrow = getTomorrow(cal);
		return isHolidayNational(yesterday, true)&&isHolidayNational(tomorrow, true);
	}
	
	public static Calendar getYesterday(Calendar cal){
		if(cal==null)return null;
		Calendar tmpCal = new GregorianCalendar();
		tmpCal.setTime(cal.getTime());
		tmpCal.add(Calendar.DATE,-1);
		return tmpCal;
	}
	public static Calendar getTomorrow(Calendar cal){
		if(cal==null)return null;
		Calendar tmpCal = new GregorianCalendar();
		tmpCal.setTime(cal.getTime());
		tmpCal.add(Calendar.DATE,+1);
		return tmpCal;
	}
	public static boolean isUruuYear(int year){
		return (year%4 == 0 && year%100 != 0 || year%400 == 0);
	}
	/*
	 * 特定の月日か？
	 * 
	 * @param month 月(1,2,...,12)
	 * @param day 月(1,2,...,31)
	 */
	private static boolean isSpecificDate(Calendar cal,int month,int day){
		if(cal==null)return false;
		return (cal!=null&&cal.get(Calendar.MONTH)+1==month&&cal.get(Calendar.DATE)==day);
	}
	/*
	 * 例：1月の第2月曜日か？--->isHolidayDow(cal,1,Calendar.MONDAY,2)
	 * 
	 * @param month 月(1,2,...,12)
	 * @param dow 曜日 Calendar.SUNDAY,...,Calendar.SATURDAY
	 * @param time 第Ndow曜日
	 */
	private static boolean isHolidayDow(Calendar cal,int month,int dow,int time){
		if(cal==null||cal.get(Calendar.MONTH)+1!=month)return false;
		if(cal.get(Calendar.DAY_OF_WEEK)!=dow)return false;
		int date = cal.get(Calendar.DATE);
		return ((date-1)/7==time-1);
	}
	
	/*
	 * 当月の稼働日を返すメソッド
	 */
	public static int getWorkdayCountInThisMonth(Calendar cal){
		return getWorkdayAndWeekendOrHolidayCountInThisMonth(cal,false,true)[0];
	}
	/*
	 * 当月の非稼働日を返すメソッド
	 */
	public static int getWeekendOrHolidayCountInThisMonth(Calendar cal){
		return getWorkdayAndWeekendOrHolidayCountInThisMonth(cal,false,true)[1];
	}
	/*
	 * 当月の本日までの稼働日を返すメソッド
	 * 
	 * @param includeToday 当日をカウントに含めるかどうか
	 */
	public static int getWorkdayCountInThisMonthUntilToday(Calendar cal,boolean includeToday){
		Calendar today = new GregorianCalendar();
		today.setTime(new Date());
		int[] vals = getWorkdayAndWeekendOrHolidayCountInThisMonth(today,true,includeToday);
		return vals[0];
		
	}
	/*
	 * 当月の本日までの非稼働日を返すメソッド
	 * 
	 * @param includeToday 当日をカウントに含めるかどうか
	 */
	public static int getWeekendOrHolidayCountInThisMonthUntilToday(Calendar cal,boolean includeToday){
		return getWorkdayAndWeekendOrHolidayCountInThisMonth(cal,true,includeToday)[1];
	}
	private static int[] getWorkdayAndWeekendOrHolidayCountInThisMonth(Calendar cal,boolean untilToday,boolean includeToday){
		
		if(cal==null)return new int[]{-1,-1};
		int workdayCount = 0;
		int weekendOrHolidayCount = 0;
		int calDay = cal.get(Calendar.DATE);
		cal.set(Calendar.DATE,1);
		int month = cal.get(Calendar.MONTH);
		while(month==cal.get(Calendar.MONTH)){
			int day = cal.get(Calendar.DATE);
			if(untilToday){
				if(includeToday){
					if(calDay<day)break;
				}else{
					if(calDay<=day)break;
				}
			}
			if(isWeekend(cal.getTime())||isHolidayNational(cal))
				weekendOrHolidayCount++;
			else
				workdayCount++;
			cal.add(Calendar.DATE, +1);
		}
		return new int[]{workdayCount,weekendOrHolidayCount};
	}
	public static int getAge(Date birth) {
		if(birth==null)return -1;
		
		Calendar birthCal = new GregorianCalendar();
		birthCal.setTime(birth);
		int bYear	 = birthCal.get(Calendar.YEAR);
		int bMonth	 = birthCal.get(Calendar.MONTH)+1;
		int bDay	 = birthCal.get(Calendar.DATE);
		Calendar cal = new GregorianCalendar();
		int cYear	 = cal.get(Calendar.YEAR);
		int cMonth	 = cal.get(Calendar.MONTH)+1;
		int cDay	 = cal.get(Calendar.DATE);

		return cYear-bYear-((bMonth>cMonth || (bMonth==cMonth&&bDay>cDay))?1:0);
	}
	/*
	 * 第3火曜→isSpecificDow(Calendar.TUESDAY,3)
	 */
	public static boolean isSpecificDow(Calendar cal,int dow, int index) {
		return isSpecificDow(cal, dow, new int[]{index});
	}
	/*
	 * 第2または第3火曜→isSpecificDow(Calendar.TUESDAY,new int[]{2,3})
	 * 
	 */
	public static boolean isSpecificDow(Calendar cal,int dow, int[] indexArray) {
		if(cal==null||cal.get(Calendar.DAY_OF_WEEK)!=dow||indexArray==null||indexArray.length<=0)return false;
		Calendar tmp = new GregorianCalendar();
		tmp.setTime(cal.getTime());
		int month = tmp.get(Calendar.MONTH);
		int count = 0;
		while(month==tmp.get(Calendar.MONTH)){
			tmp.add(Calendar.DATE,-7);
			count++;
		}
		for(int i=0;i<indexArray.length;i++){
			if(indexArray[i]==count)return true;
		}
		return false;
	}
	
	public static Calendar removeSec(Calendar cal) {
		if(cal==null)return null;
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MILLISECOND, 0);
		return cal;
	}
	public static Date removeSec(Date date) {
		if(date==null)return null;
		Calendar cal = new GregorianCalendar();
		cal.setTime(date);
		return removeSec(cal).getTime();
	}

	/*
	 * 一定時刻の間処理を停止するメソッド
	 */
	public static final void sleep(int sleepTimeInSecMin,int sleepTimeInSecMax,Long lastAccessedTimeInMillis,boolean debug){
		sleepInMillis((int)(sleepTimeInSecMin*1000+Math.random()*(sleepTimeInSecMax*1000-sleepTimeInSecMin*1000)),lastAccessedTimeInMillis,debug);
	}
	/*
	 * 一定時刻の間処理を停止するメソッド
	 */
	public static final void sleep(int sleepTimeInSec,Long lastAccessedTimeInMillis,boolean debug){
		sleepInMillis(sleepTimeInSec*1000,lastAccessedTimeInMillis,debug);
	}
	private static final void sleepInMillis(int sleepTimeInMillis,Long lastAccessedTimeInMillis,boolean debug){
		if(sleepTimeInMillis<=0)return;
		if(lastAccessedTimeInMillis==null)lastAccessedTimeInMillis = new Long(System.currentTimeMillis());
		long lastCheckedTime = System.currentTimeMillis();
		while(lastCheckedTime-lastAccessedTimeInMillis.longValue()<sleepTimeInMillis){
			long now = System.currentTimeMillis();
			//if(debug&&((int)(now-lastAccessedTimeInMillis.longValue())/1000)!=((int)(lastCheckedTime-lastAccessedTimeInMillis.longValue())/1000))
			//	System.out.print(((sleepTimeInMillis/1000)-(int)(lastCheckedTime-lastAccessedTimeInMillis.longValue())/1000)+".");
			lastCheckedTime = now;
		}
	}
}
