package st.tori.jpredict.core.util;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.TimeZone;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class StringUtil {

	private static Logger logger = Logger.getLogger(StringUtil.class.getName());

	//---------------------------------------------------------------
	// 汎用メソッド
	//---------------------------------------------------------------

	/**
	 * 必須チェックをするためのメソッド(入力されていなければtrue)
	 * @author S.Hisakawa
	 */
    public static boolean nullCheck(String str){
		if(str==null||"".equals(str))
			return true;
		return false;
	}
    public static boolean nullCheck(String[] array){
		if(array==null||array.length<=0)return true;
		return false;
	}
    /**
     * 必須チェックをするためのメソッド (nullCheck のエイリアス)
     */
    public static boolean isEmpty (String str) {
    	return nullCheck(str);
    }
    public static boolean isEmpty(String[] array) {
    	return nullCheck(array);
    }
	/**
	 * 文字列が指定された文字から成り立っているかをチェックするメソッド(例:digitCheck(pass,"0123456789abcde...."))
	 * 注意：NULLまたは空文字であればtrueを返すので、必要ならnullCheckの後に行う
	 * @param str チェック対象となる文字列
	 * @param digits 文字列を構成するベースとなるべき文字の集合
	 * @return
	 */
	public static boolean digitCheck(String str, String digits) {
		if(str==null||str.length()==0)return false;
		for(int i=0;i<str.length();i++){
			if(digits.indexOf(Character.toString(str.charAt(i)))<0)
				return true;
		}
		return false;
	}
	/**
	 * 数値チェックをするための専用メソッド(数値でなければtrue)
	 * @author S.Hisakawa
	 */
	public static boolean numCheck(String str){
		try{
			int val = Integer.parseInt(str);
		}catch(Exception ex){
			return true;
		}
		return false;
	}
	public static boolean doubleCheck(String str) {
		try{
			double val = Double.parseDouble(str);
		}catch(Exception ex){
			return true;
		}
		return false;
	}
	/**
	 * @author mariko goda
	 * Parse string to integer value or return default value.
	 * @param str : string to parse
	 * @param defaultValue : value to force if the parameter "string" is invalid format
	 * @return result
	 */
	public static int tryParseInt (String str, int defaultValue) {
		try {
			return Integer.parseInt(str);
		}
		catch (Exception ex) {
			return defaultValue;
		}
	}
	/**
	 * strの長さがlengthと等しいかチェック(等しくなければtrue)
	 * @param str
	 * @param length
	 * @return 結果
	 * @author Toshiya Seyama
	 */
	public static boolean sizeCheck(String str,int length){
		if(nullCheck(str) || str.length() != length)
			return true;
		else
			return false;
	}
	/**
	 * numの長さがlength以上になるまで先頭にdigitを追加するメソッド
	 * @param num(ex.3)
	 * @param length(ex.2)
	 * @param digit(ex."0")
	 * @return length文字以上の文字列(ex."03")
	 * @author Shingo Hisakawa
	 */
	public static String expandInt(int num,int length,String digit){
		return expandString(Integer.toString(num),length,digit);
	}
	/**
	 * numの長さがlength以上になるまで先頭にdigitを追加するメソッド
	 * @param str(ex."3")
	 * @param length(ex.2)
	 * @param digit(ex."0")
	 * @return length文字以上の文字列(ex."03")
	 * @author Shingo Hisakawa
	 */
	public static String expandString(String str,int length,String digit){
		return expandString(str,length,digit,false);
	}
	public static String expandString(String str,int length,String digit,boolean appendTail){
		if(str==null||length<=0||digit==null||digit.length()<=0)return str;
		while(str.length()<length){
			if(!appendTail)
				str = digit + str;
			else
				str += digit;
		}
		return str;
	}
	/**
	 * numの長さがlength以上になるまで末尾にdigitを追加するメソッド
	 * @param str(ex."名前")
	 * @param length(ex.5)
	 * @param digit(ex." ")
	 * @return length文字以上の文字列(ex."名前   ")
	 * @author Toshiya Seyama
	 */
	public static String leftExpandString(String str,int length,String digit){
		while(str.length()<length)str += digit;
		return str;
	}
	public static String rightExpandString(String str,int length,String digit){
		while(str.length()<length)str = digit+str;
		return str;
	}
	/**
	 * strの長さがmin以上max以下にあるかチェック(範囲外であればtrue)
	 * @param str
	 * @param min
	 * @param max
	 * @return 結果
	 * @author Toshiya Seyama
	 */
	public static boolean sizeCheck(String str,int min,int max){
		if(nullCheck(str) || str.length() < min || str.length() > max)
			return true;
		else
			return false;
	}
	/**
	 * strのbyte数がmin以上max以下にあるかチェック(範囲外であればtrue)
	 * @param str
	 * @param min
	 * @param max
	 * @return 結果
	 * @author Eisuke Jindo
	 */
	public static boolean byteCheck(String str,int min,int max){
		if(str.getBytes().length< min || str.getBytes().length > max)
			return true;
		else
			return false;
	}
	/**
	 * 日付として正しいか判別(日付として不適切であればtrue)
	 * 
	 * @param year
	 * @param month
	 * @param day
	 * @return 結果
	 * @author Shingo Hisakawa
	 */
	public static boolean dateCheck(String year,String month,String day) {
		//数値でないものが含まれていないか
		if(numCheck(year)||numCheck(month)||numCheck(day)){
			return true;
		}
		
		int yearInt = forceInt(year);
		int monthInt = forceInt(month);
		int dayInt = forceInt(day);
		
		//月・日が範囲外でないか
		if(yearInt<1900||monthInt<1||monthInt>12||dayInt<1||dayInt>31){
			return true;
		}
		//日付として不正でないか
		GregorianCalendar tmp = new GregorianCalendar(yearInt, monthInt - 1, dayInt);
		if (yearInt != tmp.get(GregorianCalendar.YEAR) 
		|| monthInt != tmp.get(GregorianCalendar.MONTH) + 1
		|| dayInt != tmp.get(GregorianCalendar.DATE)){
			return true;
		}

		return false;
	}
	/**
	 * 日時として正しいか判別(日時として不適切であればtrue)
	 * 
	 * @param year
	 * @param month
	 * @param day
	 * @param hour
	 * @param minute
	 * @return 結果
	 * @author Y.Kobayashi
	 */
	public static boolean dateCheck(String year,String month,String day,String hour,String minute) {
		//数値でないものが含まれていないか
		if(numCheck(year)||numCheck(month)||numCheck(day)||numCheck(hour)||numCheck(minute))
			return true;
		
		int yearInt = forceInt(year);
		int monthInt = forceInt(month);
		int dayInt = forceInt(day);
		int hourInt = forceInt(hour);
		int minuteInt = forceInt(minute);
		
		//月・日が範囲外でないか
		if(yearInt<1900||monthInt<1||monthInt>12||dayInt<1||dayInt>31)
			return true;
		//時・分が範囲外でないか
		if(hourInt<0 || 23<hourInt || minuteInt<0 || 59<minuteInt)
			return true;
		//日時として不正でないか
		GregorianCalendar tmp = new GregorianCalendar(yearInt, monthInt - 1, dayInt, hourInt, minuteInt);
		if (yearInt != tmp.get(GregorianCalendar.YEAR) 
		|| monthInt != tmp.get(GregorianCalendar.MONTH) + 1
		|| dayInt != tmp.get(GregorianCalendar.DATE)
		|| hourInt != tmp.get(GregorianCalendar.HOUR_OF_DAY)
		|| minuteInt != tmp.get(GregorianCalendar.MINUTE))
			return true;

		return false;
	}
	/**
	 * 全角カナ入力であるかチェック(全角カナ＋半角英数＋記号「()・．ー−」＋空白（全角・半角）であればtrue)
	 * @param str
	 * @return 結果
	 * @author Shingo Hisakawa
	 */
	public static boolean isZenKana(String str){
		return isOnlyMadeFrom(str,"1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyzアイウエオカキクケコサシスセソタチツテトナニヌネノハヒフヘホマミムメモヤユヨラリルレロワヰヱヲンガギグゲゴザジズゼゾダヂヅデドバビブベボパピプペポヴァィゥェォャュョッヵヶ()（）・．ー− 　");
	}	      
	/**
	 * 指定された文字列で構成されているかチェック(指定された文字列から構成されていればtrue)
	 * @param str
	 * @param allowedStr 指定文字列
	 * @return 結果
	 * @author Shingo Hisakawa
	 */
	public static boolean isOnlyMadeFrom(String str,String allowedStr){
		boolean okFlg = true;
		String sub = null;
		for (int i = 0; i < str.length(); i++) {
			sub = str.substring(i, i + 1);
			if(allowedStr.indexOf(sub)==-1){
				okFlg = false;
				break;
			}
		}
		return okFlg;
	}	      
	/**
	 * 指定された文字列を含んでいるかチェック(指定された文字列を含んでいればtrue)
	 * @param str
	 * @param partsStr 指定文字列
	 * @return 結果
	 * @author Shingo Hisakawa
	 */
	public static boolean contains(String str,String partsStr){
		boolean okFlg = false;
		String sub = null;
		for (int i = 0; i < str.length(); i++) {
			sub = str.substring(i, i + 1);
			if(partsStr.indexOf(sub)!=-1){
				okFlg = true;
				break;
			}
		}
		return okFlg;
	}	      
	/**
	 * 指定された文字列で構成されているかチェック(指定された以外の文字列を含んでいればfalse)
	 * @param str
	 * @param partsStr 指定文字列
	 * @return 結果
	 * @author Shingo Hisakawa
	 */
	public static boolean consists(String str,String partsStr){
		boolean okFlg = true;
		String sub = null;
		for (int i = 0; i < str.length(); i++) {
			sub = str.substring(i, i + 1);
			if(partsStr.indexOf(sub)==-1){
				okFlg = false;
				break;
			}
		}
		return okFlg;
	}	      
    /**
	 * 文字列を強制的にint化するためのメソッド(数値形式でなければ-1)
	 * @author S.Hisakawa
	 */
	public static int forceInt(String str){
		return forceInt(str,-1);
	}
	public static int forceInt(String str, int valOnError) {
		int tmp = valOnError;
		if(str==null||str.length()==0)return tmp;
		try{
			tmp = Integer.parseInt(str);
		}catch(Exception e){
			tmp = valOnError;
		}
		return tmp;
	}
    /**
	 * 文字列を強制的にInteger化するためのメソッド(数値形式でなければnull)
	 * @author S.Hisakawa
	 */
	public static Integer forceIntObject(String str){
		Integer tmp = null;
		if(str==null||str.length()==0)return tmp;
		try{
			tmp = new Integer(str);
		}catch(Exception e){
			tmp = null;
		}
		return tmp;
	}
    /**
	 * 文字列配列を強制的にint配列化するためのメソッド(数値形式でないものがあればnull)
	 * @author S.Hisakawa
	 */
	public static int[] forceInt(String[] str){
		int[] tmp = null;
		if(str==null)return tmp;
		try{
			tmp = new int[str.length];
			for(int i=0;i<str.length;i++)
				tmp[i] = Integer.parseInt(str[i]);
		}catch(Exception e){
			return null;
		}
		return tmp;
	}
    /**
	 * 文字列配列を強制的にInteger配列化するためのメソッド(数値形式でないものがあればnull)
	 * @author S.Hisakawa
	 */
	public static Integer[] forceIntObject(String[] str){
		Integer[] tmp = null;
		if(str==null)return tmp;
		try{
			tmp = new Integer[str.length];
			for(int i=0;i<str.length;i++)
				tmp[i] = new Integer(str[i]);
		}catch(Exception e){
			return null;
		}
		return tmp;
	}
    /**
	 * 文字列を強制的にshort化するためのメソッド(数値形式でなければ-1)
	 * @author S.Hisakawa
	 */
	public static short forceShort(String str){
		short tmp = -1;
		if(str==null||str.length()==0)return tmp;
		try{
			tmp = Short.parseShort(str);
		}catch(Exception e){
			tmp = -1;
		}
		return tmp;
	}
    /**
	 * 文字列を強制的にShort化するためのメソッド(数値形式でなければnull)
	 * @author S.Hisakawa
	 */
	public static Short forceShortObject(String str){
		Short tmp = null;
		if(str==null||str.length()==0)return tmp;
		try{
			tmp = new Short(str);
		}catch(Exception e){
			tmp = null;
		}
		return tmp;
	}
    /**
	 * 文字列配列を強制的にShort配列化するためのメソッド(数値形式でないものがあればnull)
	 * @author S.Hisakawa
	 */
	public static Short[] forceShortObject(String[] str){
		Short[] tmp = null;
		if(str==null)return tmp;
		try{
			tmp = new Short[str.length];
			for(int i=0;i<str.length;i++)
				tmp[i] = new Short(str[i]);
		}catch(Exception e){
			return null;
		}
		return tmp;
	}
    /**
	 * 文字列を強制的にdouble化するためのメソッド(数値形式でなければ-1.0)
	 * @author S.Hisakawa
	 */
	public static double forceDouble(String str){
		return forceDouble(str,-1.0);
	}
	public static double forceDouble(String str, double valOnError){
		double tmp = valOnError;
		if(str==null||str.length()==0)return tmp;
		try{
			tmp = Double.parseDouble(str);
		}catch(Exception e){
			tmp = valOnError;
		}
		return tmp;
	}
    /**
	 * 文字列を強制的にfloat化するためのメソッド(数値形式でなければ-1.0)
	 * @author S.Hisakawa
	 */
	public static float forceFloat(String str){
		return forceFloat(str,(float)-1.0);
	}
	public static float forceFloat(String str, float valOnError){
		float tmp = valOnError;
		if(str==null||str.length()==0)return tmp;
		try{
			tmp = Float.parseFloat(str);
		}catch(Exception e){
			tmp = valOnError;
		}
		return tmp;
	}
    /**
	 * 文字列を強制的にDouble化するためのメソッド(数値形式でなければnull)
	 * @author S.Hisakawa
	 */
	public static Double forceDoubleObject(String str){
		Double tmp = null;
		if(str==null||str.length()==0)return tmp;
		try{
			tmp = new Double(str);
		}catch(Exception e){
			tmp = null;
		}
		return tmp;
	}
    /**
	 * 文字列配列を強制的にDouble配列化するためのメソッド(数値形式でないものがあればnull)
	 * @author S.Hisakawa
	 */
	public static Double[] forceDoubleObject(String[] str){
		Double[] tmp = null;
		if(str==null)return tmp;
		try{
			tmp = new Double[str.length];
			for(int i=0;i<str.length;i++)
				tmp[i] = new Double(str[i]);
		}catch(Exception e){
			return null;
		}
		return tmp;
	}
    /**
	 * 文字列を強制的にFloat化するためのメソッド(数値形式でなければnull)
	 * @author S.Hisakawa
	 */
	public static Float forceFloatObject(String str){
		Float tmp = null;
		if(str==null||str.length()==0)return tmp;
		try{
			tmp = new Float(str);
		}catch(Exception e){
			tmp = null;
		}
		return tmp;
	}
    /**
	 * 文字列配列を強制的にFloat配列化するためのメソッド(数値形式でないものがあればnull)
	 * @author S.Hisakawa
	 */
	public static Float[] forceFloatObject(String[] str){
		Float[] tmp = null;
		if(str==null)return tmp;
		try{
			tmp = new Float[str.length];
			for(int i=0;i<str.length;i++)
				tmp[i] = new Float(str[i]);
		}catch(Exception e){
			return null;
		}
		return tmp;
	}
    /**
	 * 文字列を強制的にlong化するためのメソッド(数値形式でなければ0)
	 * @author S.Hisakawa
	 */
	public static long forceLong(String str){
		return forceLong(str,0);
	}
	public static long forceLong(String str, long valOnError) {
		long tmp = valOnError;
		if(str==null||str.length()==0)return tmp;
		try{
			tmp = Long.parseLong(str);
		}catch(Exception e){
			tmp = valOnError;
		}
		return tmp;
	}
    /**
	 * 文字列を強制的にInteger化するためのメソッド(数値形式でなければnull)
	 * @author S.Hisakawa
	 */
	public static Integer forceIntegerObject(String str){
		Integer tmp = null;
		if(str==null||str.length()==0)return tmp;
		try{
			tmp = new Integer(str);
		}catch(Exception e){
			tmp = null;
		}
		return tmp;
	}
    /**
	 * 文字列配列を強制的にInteger配列化するためのメソッド(数値形式でないものがあればnull)
	 * @author S.Hisakawa
	 */
	public static Integer[] forceIntegerObject(String[] str){
		Integer[] tmp = null;
		if(str==null)return tmp;
		try{
			tmp = new Integer[str.length];
			for(int i=0;i<str.length;i++)
				tmp[i] = new Integer(str[i]);
		}catch(Exception e){
			return null;
		}
		return tmp;
	}
    /**
	 * 文字列を強制的にBoolean化するためのメソッド(bool形式でなければnull)
	 * @author S.Hisakawa
	 */
	public static Boolean forceBooleanObject(String str){
		if(str==null||str.length()==0)
			return null;
		
		str = str.toLowerCase();
		if("y".equals(str)||"true".equals(str))
			return new Boolean(true);
		else if("n".equals(str)||"false".equals(str))
			return new Boolean(false);
		else
			return null;
	}
    /**
	 * 文字列配列を強制的にBoolean配列化するためのメソッド(数値形式でないものがあればnull)
	 * @author S.Hisakawa
	 */
	public static Boolean[] forceBooleanObject(String[] str){
		Boolean[] tmp = null;
		if(str==null)return tmp;
		try{
			tmp = new Boolean[str.length];
			for(int i=0;i<str.length;i++)
				tmp[i] = new Boolean(str[i]);
		}catch(Exception e){
			return null;
		}
		return tmp;
	}
    /**
	 * 文字列を強制的に日付化するためのメソッド(日付形式でなければnull)
	 * @author S.Hisakawa
	 */
	public static Date forceDate(String year,String month,String day){
		if(dateCheck(year,month,day)){
			return null;
		}
		return (new GregorianCalendar(forceInt(year),forceInt(month)-1,forceInt(day))).getTime();
	}
    /**
	 * 文字列を強制的に日時化するためのメソッド(日時形式でなければnull)
	 * @author Y.Kobayashi
	 */
	public static Date forceDate(String year,String month,String day,String hour,String minute){
		if(dateCheck(year,month,day,hour,minute))
			return null;
		
		return (new GregorianCalendar(forceInt(year),forceInt(month)-1,forceInt(day),forceInt(hour),forceInt(minute))).getTime();
	}

	/**
	 * 少数第一位までパーセントを計算するメソッド
	 * @author S.Hisakawa
	 * @param numerator 分子
	 * @param denominator 分母
	 */
	public static String getPercent(String numerator,String denominator) {
		if (numCheck(numerator)||numCheck(denominator))
			return "0.0";
		return getPercent(Integer.parseInt(numerator),Integer.parseInt(denominator));
	}
	/**
	 * 少数第一位までパーセントを計算するメソッド
	 * @author S.Hisakawa
	 * @param numerator 分子
	 * @param denominator 分母
	 */
	public static String getPercent(int numerator,int denominator) {
		return getRatio(numerator*100,denominator);
	}
	public static String getPercent(double numerator,double denominator) {
		return getRatio(numerator*100,denominator);
	}
	/**
	 * 小数第一位まで割合を計算するメソッド
	 * @author S.Hisakawa
	 * @param numerator 分子
	 * @param denominator 分母
	 */
	public static String getRatio(String numerator,String denominator) {
		if (numCheck(numerator)||numCheck(denominator))
			return "0.0";
		return getRatio(Integer.parseInt(numerator),Integer.parseInt(denominator));
	}
	/**
	 * 小数第一位まで割合を計算するメソッド
	 * @author S.Hisakawa
	 * @param numerator 分子
	 * @param denominator 分母
	 */
	public static String getRatio(int numerator,int denominator) {
		return getRatio((double)numerator,(double)denominator);
	}
	public static String getRatio(double numerator,double denominator) {
		if (numerator==0||denominator==0)
			return "0.0";
		String ratio = String.valueOf((int)(numerator * 10.0 / denominator));
		if (ratio.length() < 2)ratio = "0" + ratio;
		ratio = ratio.substring(0, ratio.length() - 1) + "." + ratio.substring(ratio.length() - 1);
		return ratio;
	}
	
	/*
	 * 指定された文字列が正規表現にマッチするかを判定するメソッド
	 */
	public static boolean isRegexp(String str,String regexp){
		if(str==null||regexp==null)return false;
		Pattern pattern = Pattern.compile(regexp, Pattern.CASE_INSENSITIVE);
		return isRegexp(str,pattern);
	}
	/*
	 * 指定された文字列が正規表現にマッチするかを判定するメソッド
	 */
	public static boolean isRegexp(String str,Pattern pattern){
		Matcher objMch = pattern.matcher(str);
		if(objMch.matches()){
			//bnd.info("Util.isRegexp:TRUE:"+str+" <-> "+regexp);
			return true;
		}else{
			//bnd.info("Util.isRegexp:FALSE:"+str+" <-> "+regexp);
			return false;
		}
	}

	/*
	 * URLエンコードするメソッド
	 */
	public static String urlEncode(String str,String encode) throws UnsupportedEncodingException{
		return URLEncoder.encode(str,encode);
	}
	/*
	 * URLデコードするメソッド
	 */
	public static String urlDecode(String str,String encode) throws UnsupportedEncodingException{
		return URLDecoder.decode(str,encode);
	}

	public static String formatAsXmlNodeValue(String val){
		if(val==null||"null".equals(val))return "";
		val = val.replaceAll("&amp;", "&");
		val = val.replaceAll("&", "&amp;");
		val = val.replaceAll("<","&lt;");
		val = val.replaceAll(">","&gt;");
		val = val.replaceAll("\"","&quot;"); 
		return val;
	}

	public static final String formatDate (Date date, String formatStr) {
		return formatDate(date,formatStr,null);
	}
	public static final String formatDate (Date date, String formatStr,TimeZone tz) {
		return formatDate(date,formatStr,tz,null);
	}
	public static final String formatDate (Date date, String formatStr,TimeZone tz,Locale locale) {
		if (date==null) return null;
		SimpleDateFormat format = (locale!=null)?new SimpleDateFormat(formatStr,locale):new SimpleDateFormat(formatStr);
		if(tz!=null)format.setTimeZone(tz);
		return format.format(date);
	}
	public static final String formatDateAsTimeInMillis (Date date) {
		return (date==null)?null:String.valueOf(date.getTime());
	}
	/**
	 * SQLで問題になる文字をエスケープする
	 * 
	 * @param str
	 * @return エスケープ済み文字列
	 * @author Toshiya Seyama
	 */
	public static String sqlEscape(String str){
		if(str==null)return null;
		str = str.replaceAll("\'","\'\'");
		str = str.replaceAll("\\\\","\\\\\\\\\\\\\\\\");
		str = str.replaceAll("%","\\\\\\\\%");
		//str = str.replaceAll("_","\\\\\\\\_");
		return str; 
	}
	public static String concat(String[] array, String sep) {
		if(array==null||array.length<=0)return null;
		StringBuffer buf = new StringBuffer();
		for(int i=0;i<array.length;i++){
			if(i>0)buf.append(sep);
			buf.append(array[i]);
		}
		return buf.toString();
	}
    /**
     * 文字列がASCIIコード(制御文字は除く)であるかどうかを返すメソッド
     * @param str
     * @return
     */
    public static boolean isAscii(String str) {
		if(str==null||str.length()==0)return false;
		char ch;
		for (int i = 0; i < str.length(); i++) {
			ch = str.charAt(i);
			if (!(ch >= 0x20 && ch <= 0x7E)) {
				return false;
			}
		}
		return true;
	}
}