package st.tori.jpredict.levistone.activity;

import st.tori.android.core.activity.BluetoothActivity;
import st.tori.jpredict.levistone.activity.MainActivity.Offsets;
import st.tori.jpredict.levistone.db.LevistoneData;
import st.tori.jpredict.levistone.db.LevistoneData.YValues;
import st.tori.jpredict.levistone.exception.CalibrationRequiredException;
import st.tori.jpredict.levistone.orm.OperandCalibrate;
import st.tori.jpredict.levistone.orm.OperandRotate;
import st.tori.jpredict.levistone.orm.RotateStatus;
import android.util.Log;

public abstract class AbstractCalibrateActivity extends BluetoothActivity {

	private static final String TAG = "AbstractCalibrateActivity";
	
	protected abstract void writeIfPossible();
	
    protected Offsets offsets = new Offsets();
    protected RotateStatus rotateStatus = new RotateStatus(0,90);
    protected OperandRotate operandRotate = new OperandRotate(rotateStatus,offsets);
    protected OperandCalibrate operandCalibrateAaz = new OperandCalibrate(OperandCalibrate.TARGET_A_AZ);
    protected OperandCalibrate operandCalibrateBaz = new OperandCalibrate(OperandCalibrate.TARGET_B_AZ);
    protected OperandCalibrate operandCalibrateAel = new OperandCalibrate(OperandCalibrate.TARGET_A_EL);
    protected OperandCalibrate operandCalibrateBel = new OperandCalibrate(OperandCalibrate.TARGET_B_EL);

    private static final boolean dispatchCalibrationForFirst = true;
    protected boolean hasCoInDB(){
    	LevistoneData data = new LevistoneData(this);
    	try{
	    	return (data.getYValues()!=null);
    	}finally{
    		data.close();
    	}
    	
    }
    protected void updateCoWithDB(boolean throwException) throws CalibrationRequiredException {
    	YValues obj = null;
    	LevistoneData data = new LevistoneData(this);
    	try{
        	obj = new LevistoneData(this).getYValues();
    	}finally{
    		data.close();
    	}
    	y_a_0	 = (obj!=null)?obj.getYA0():((!dispatchCalibrationForFirst)?6:Integer.MAX_VALUE);
    	y_a_360	 = (obj!=null)?obj.getYA360():((!dispatchCalibrationForFirst)?105:Integer.MAX_VALUE);
    	y_e_0	 = (obj!=null)?obj.getYE0():((!dispatchCalibrationForFirst)?4:Integer.MAX_VALUE);
    	y_e_90	 = (obj!=null)?obj.getYE90():((!dispatchCalibrationForFirst)?81:Integer.MAX_VALUE);
	    updateCo(y_a_0,y_a_360,y_e_0,y_e_90,throwException);
    }
    protected int y_a_0		 = Integer.MAX_VALUE;
    protected int y_a_360	 = Integer.MAX_VALUE;
    protected int y_e_0		 = Integer.MAX_VALUE;
    protected int y_e_90	 = Integer.MAX_VALUE;
    protected double a_az;
    protected double b_az;
    protected double a_el;
    protected double b_el;
    protected void updateCo(int y_a_0,int y_a_360,int y_e_0,int y_e_90,boolean throwException) throws CalibrationRequiredException {
    	this.y_a_0 = y_a_0;
    	this.y_a_360 = y_a_360;
    	this.y_e_0 = y_e_0;
    	this.y_e_90 = y_e_90;
    	if(y_a_0==Integer.MAX_VALUE||y_e_90==Integer.MAX_VALUE||y_a_360==Integer.MAX_VALUE||y_e_0==Integer.MAX_VALUE){
    		if(throwException)
    			throw new CalibrationRequiredException();
    		else
    			return;
    	}
    	YValues obj = new YValues(y_a_0,y_a_360,y_e_0,y_e_90);
    	LevistoneData data = new LevistoneData(this);
    	try{
        	data.saveYValues(obj);
    	}finally{
    		data.close();
    	}
		this.a_az = (y_a_360-y_a_0)/360.0;
		this.b_az = y_a_0;
		this.a_el = (y_e_90-y_e_0)/90.0;
		this.b_el = y_e_0;
		Log.d(TAG,"a_az="+a_az+",b_az="+b_az+",a_el="+a_el+",b_el="+b_el);
		operandCalibrateAaz.write(a_az);
		operandCalibrateBaz.write(b_az);
		operandCalibrateAel.write(a_el);
		operandCalibrateBel.write(b_el);
		operandRotate.set(a_az,b_az,a_el,b_el);
		operandRotate.moveTo(0,90,false);
		writeIfPossible();
    }

}
