package st.tori.jpredict.levistone.activity;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import st.tori.android.core.util.DialogUtil;
import st.tori.jpredict.core.orm.GpsPositionInterface;
import st.tori.jpredict.core.orm.SimpleGpsPosition;
import st.tori.jpredict.levistone.bizlogic.ReadWriteThread;
import st.tori.jpredict.levistone.bizlogic.ReadWriteThread.ReadListener;
import st.tori.jpredict.levistone.exception.CalibrationRequiredException;
import st.tori.jpredict.levistone.model.SatListContainer;
import st.tori.jpredict.levistone.model.Satellite;
import st.tori.jpredict.levistone.model.course.SatCourse;
import st.tori.jpredict.levistone.model.course.SatCoursePointWithAzEl;
import st.tori.jpredict.levistone.network.tsv.SatCourseWithAzElParser;
import st.tori.jpredict.levistone.network.tsv.SatListParser;
import st.tori.jpredict.levistone.network.tsv.TimeParser;
import st.tori.jpredict.levistone.orm.Operand;
import st.tori.jpredict.levistone.orm.OperandSequence;
import st.tori.jpredict.levistone.orm.OperandSwitch;
import st.tori.jpredict.levistone.orm.OperandTrack;
import st.tori.jpredict.levistone.orm.SwitchStatus;
import android.content.Context;
import android.content.Intent;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.speech.tts.TextToSpeech;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.LinearLayout;
import android.widget.TableLayout;
import android.widget.TextView;
import android.widget.ToggleButton;

public class MainActivity extends AbstractCalibrateActivity implements OnCheckedChangeListener, OnTouchListener, LocationListener, SensorEventListener, TextToSpeech.OnInitListener {
	
	private static final String TAG = "MainActivity";
	private static final double LASER_ON_THRETHOLD_IN_DEG = 20.0;
	
	private Handler handler = new Handler();
	
	private LocationManager lManager;
	private SensorManager mSensorManager;
	private Sensor mCompass;
    
	private TextToSpeech tts;
	private boolean isTextToSpeechActivated(){	return false;	}
	private float ttsPitch = 1.0f;
	private float ttsRate = 1.0f;

	private LinearLayout mainBackground;
	private TableLayout debugBox;
	private TextView debugTextBT;
	private TextView debugTextCo;
	private TextView debugTextTS;
	private TextView debugTextLocation;
	private TextView debugTextTime;
	private TextView debugTextSatList;
	private TextView debugTextSatOrientation;
	private TextView debugTextCompass;
	
	private TextView statusText;
	private TableLayout latLonBox;
	private TextView latitudeText;
	private TextView longitudeText;
	private TextView azimuthText;
	private TextView elevationText;
	private TextView altitudeText;
	private TextView slantRangeText;
	private TextView velocitySecText;
	private TextView velocityHourText;
	
	private ToggleButton modeToggle;
	private ToggleButton blueToggle;
	private ToggleButton laserToggle;
	private ToggleButton redToggle;
	private Button laserActivateButton;
    
	//-----------------
	//	基本要素
	//-----------------
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	Log.d(TAG,"%%%onCreate START");
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        clearAbort();
        
        mainBackground = (LinearLayout)findViewById(R.id.mainBackground);
    	mainBackground.setBackgroundResource(R.drawable.laputa_normal);
    	debugBox = (TableLayout)findViewById(R.id.debugBox);
    	debugTextBT = (TextView)findViewById(R.id.debugTextBT);
    	debugTextCo = (TextView)findViewById(R.id.debugTextCo);
    	debugTextTS = (TextView)findViewById(R.id.debugTextTS);
    	debugTextLocation = (TextView)findViewById(R.id.debugTextLocation);
    	debugTextTime = (TextView)findViewById(R.id.debugTextTime);
    	debugTextSatList = (TextView)findViewById(R.id.debugTextSatList);
    	debugTextSatOrientation = (TextView)findViewById(R.id.debugTextSatOrientation);
    	debugTextCompass = (TextView)findViewById(R.id.debugTextCompass);
    	statusText = (TextView)findViewById(R.id.statusText);
    	latLonBox = (TableLayout)findViewById(R.id.latlonBox);
    	latitudeText = (TextView)findViewById(R.id.latitudeText);
    	longitudeText = (TextView)findViewById(R.id.longitudeText);
    	azimuthText = (TextView)findViewById(R.id.azimuthText);
    	elevationText = (TextView)findViewById(R.id.elevationText);
    	altitudeText = (TextView)findViewById(R.id.altitudeText);
    	slantRangeText = (TextView)findViewById(R.id.slantRangeText);
    	velocitySecText = (TextView)findViewById(R.id.velocitySecText);
    	velocityHourText = (TextView)findViewById(R.id.velocityHourText);
    	modeToggle = (ToggleButton)findViewById(R.id.modeToggle);
    	blueToggle = (ToggleButton)findViewById(R.id.blueToggle);
    	laserToggle = (ToggleButton)findViewById(R.id.laserToggle);
    	redToggle = (ToggleButton)findViewById(R.id.redToggle);
    	laserActivateButton = (Button)findViewById(R.id.laserActivateButton);
    	
    	Satellite selectedTarget = getIntent().getParcelableExtra(TargetListActivity.EXTRA_NAME_TARGET_SAT);
    	if(selectedTarget!=null){
    		Log.d(TAG, "selectedTarget="+selectedTarget);
    		setTarget(selectedTarget);
    	}else if(target==null){
    		setTarget(getTarget());
    	}
    	if(!offsets.isTimeCorrected())
    		setTimeOffset();
    	
    	if(isTextToSpeechActivated())
    		tts = new TextToSpeech(this, this);
        lManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
        lManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, this);
        mSensorManager = (SensorManager)getSystemService(SENSOR_SERVICE);
        mCompass = mSensorManager.getDefaultSensor(Sensor.TYPE_ORIENTATION);

        modeToggle.setOnCheckedChangeListener(this);
    	blueToggle.setOnCheckedChangeListener(this);
    	laserToggle.setOnCheckedChangeListener(this);
    	redToggle.setOnCheckedChangeListener(this);
    	laserActivateButton.setOnTouchListener(this);
		
    	blueToggle.setEnabled(false);
    	laserToggle.setEnabled(false);
    	redToggle.setEnabled(false);
    	
        if(!hasCoInDB()){
        	Log.d(TAG,"%%%onCreate Dispatch to CalibrateActivity");
			speak("Entering calibrate mode");
			abortAllOperands();
			Intent intent = new Intent(this, CalibrateActivity.class);
			Log.d(TAG,"startActivity1");
			startActivity(intent);
        }else{
        	Log.d(TAG,"%%%onCreate No Dispatch to CalibrateActivity");
            onCreateBluetooth(savedInstanceState);
        }
        
    	Log.d(TAG,"%%%onCreate END");
    }
	@Override
	protected void onResume() {
    	Log.d(TAG,"%%%onResume START");
		super.onResume();
		mSensorManager.registerListener(this, mCompass, SensorManager.SENSOR_DELAY_NORMAL);
    	Log.d(TAG,"%%%onResume END");
	}
	@Override
    protected void onPause() {
    	Log.d(TAG,"%%%onPause START");
    	super.onPause();
    	mSensorManager.unregisterListener(this);
    	//if(thread!=null)
    	//	thread.stop();
    	Log.d(TAG,"%%%onPause END");
    }
	@Override
	public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
    	Log.d(TAG,"%%%onCheckedChanged START:isAuto()="+switchStatus.isAuto());
		if(buttonView.getId()==modeToggle.getId()){
			if(isChecked){
				//AUTO
				blueToggle.setChecked(true);
				blueToggle.setEnabled(false);
				laserToggle.setChecked(true);
				laserToggle.setEnabled(false);
				redToggle.setChecked(true);
				redToggle.setEnabled(false);
			}else{
				//MANUAL
				blueToggle.setChecked(false);
				blueToggle.setEnabled(true);
				laserToggle.setChecked(false);
				laserToggle.setEnabled(true);
				redToggle.setChecked(false);
				redToggle.setEnabled(true);
			}
	    	Log.d(TAG,"%%%onCheckedChanged modeToggled");
	    	switchStatus.setAuto(isChecked);
			writeNow(new OperandSwitch(switchStatus, SwitchStatus.LED_MODE_OFF, SwitchStatus.LASER_MODE_OFF, SwitchStatus.LED_MODE_OFF));
		}else if(buttonView.getId()==blueToggle.getId()&&!switchStatus.isAuto()){
			int blueMode = (isChecked)?SwitchStatus.LED_MODE_ON:SwitchStatus.LED_MODE_OFF;
	    	Log.d(TAG,"%%%onCheckedChanged blueToggled");
			writeNow(new OperandSwitch(switchStatus, blueMode, switchStatus.getLaserMode(), switchStatus.getRedMode()));
		}else if(buttonView.getId()==laserToggle.getId()&&!switchStatus.isAuto()){
	    	Log.d(TAG,"%%%onCheckedChanged laserToggled");
		}else if(buttonView.getId()==redToggle.getId()&&!switchStatus.isAuto()){
			int redMode = (isChecked)?SwitchStatus.LED_MODE_ON:SwitchStatus.LED_MODE_OFF;
	    	Log.d(TAG,"%%%onCheckedChanged redToggled");
			writeNow(new OperandSwitch(switchStatus, switchStatus.getBlueMode(), switchStatus.getLaserMode(), redMode));
		}
    	Log.d(TAG,"%%%onCheckedChanged END");
	}
    
	//-----------------
	//	BT関連
	//-----------------
	
    @Override
    protected boolean isBluetoothInactivated(){
    	return false;
    }
    @Override
    protected String getBluetoothTargetDeviceName(){
    	return "ARDUINOBT";
    }
    @Override
	protected String getBluetoothTargetPassKey(){
		return "12345";
	}
    @Override
    protected void onBluetoothConnectResult(int result){
    	Log.d(TAG,"%%%onBluetoothConnectResult START");
    	if(result==RESULT_OK){
    		debugTextBT.setText(btDevice.getName());
    		try {
    			updateCoWithDB(true);
    			debugTextCo.setText(neaten(a_az)+"/"+neaten(b_az)+"/"+neaten(a_el)+"/"+neaten(b_el));
    	    	switchStatus.setAuto(true);
    			writeNow(new OperandSwitch(switchStatus, SwitchStatus.LED_MODE_OFF, SwitchStatus.LASER_MODE_OFF, SwitchStatus.LED_MODE_OFF));
    			trackIfPossible();
    		} catch (CalibrationRequiredException e) {
				speak("Entering calibrate mode");
				abortAllOperands();
				closeBluetoothSocket(true);
				Intent intent = new Intent(this, CalibrateActivity.class);
				intent.putExtra(EXTRA_NAME_BT_DEVICE, getBluetoothDevice());
				Log.d(TAG,"startActivity2");
				startActivity(intent);
    		}
    	}else{
    		DialogUtil.showSimpleDialog(this, "Error", "Cannot connect to levistone.Please make sure that it's close and turned on.");
    		abortAllOperands();
    	}
    	Log.d(TAG,"%%%onBluetoothConnectResult END");
    }

	//-----------------
	//	駆動系
	//-----------------
    
    protected Thread thread;
    protected Operand currentOperand = null;
    
    protected SwitchStatus switchStatus = new SwitchStatus(true,SwitchStatus.LED_MODE_ON,SwitchStatus.LASER_MODE_ON,SwitchStatus.LED_MODE_ON);
    protected OperandSequence operandSequence = new OperandSequence();
    protected OperandSwitch operandSwitch = new OperandSwitch(switchStatus, switchStatus.getBlueMode(), switchStatus.getLaserMode(), switchStatus.getRedMode());
    protected OperandTrack operandTrack = new OperandTrack(operandRotate,operandSwitch);
    protected Operand[] operands = new Operand[]{
    	operandSequence,operandCalibrateAaz,operandCalibrateBaz,operandCalibrateAel,operandCalibrateBel,
    	operandRotate,operandSwitch,operandTrack};
    
    private boolean abortAllOperands = false;
    protected void abortAllOperands(){
    	suspendBluetooth = true;
    	abortAllOperands = true;
    	for(int i=0;i<operands.length;i++)
    		operands[i].execting();
    	currentOperand = null;
    	if(thread!=null)
    		thread.stop();
		thread = null;
    }
    protected void clearAbort(){
    	suspendBluetooth = false;
    	abortAllOperands = false;
    }
    
    @Override
    protected void writeIfPossible(){
    	//Log.d(TAG,"%%%writeIfPossible START");
    	if(abortAllOperands){
    		abortAllOperands();
        	Log.d(TAG,"%%%writeIfPossible abortAllOperands");
        	return;
    	}
    	if(currentOperand!=null){
    		Log.d(TAG,"writeIfPossible:exit by lock");
    		return;
    	}
    	for(int i=0;i<operands.length;i++){
    		if(!operands[i].isWaiting())continue;
        	currentOperand = operands[i];
        	openBluetoothSocketIfClosed();
    		thread = new ReadWriteThread(getBluetoothSocket(),offsets,currentOperand,new MyListener());
    		thread.start();
    		return;
    	}
    	//Log.d(TAG,"%%%writeIfPossible NO operand waiting.close socket");
    	closeBluetoothSocket(false);
    	//Log.d(TAG,"%%%writeIfPossible END");
    }
    protected void writeNow(Operand operand){
    	if(operand==null)return;
    	openBluetoothSocketIfClosed();
		thread = new ReadWriteThread(getBluetoothSocket(),offsets,operand,null);
		thread.start();
    }
    
	private class MyListener implements ReadListener {
		@Override
		public void read(String cmdStr,String response) {
			Log.d(TAG,"MyListener.read:"+cmdStr);
			if(currentOperand!=null){
				if(currentOperand instanceof OperandSequence){
					if(currentOperand instanceof OperandTrack){
						autoSwitch();
						handler.post(new Runnable() {
							@Override
							public void run() {
								SatCoursePointWithAzEl point = ((OperandTrack)currentOperand).getSatCoursePointWithAzEl();
								int spanInMillis = (int)(getCurrentTimeMillis()-trackStartTime);
								String span;
								if(spanInMillis<1*60*1000)
									span = (spanInMillis/1000.0)+"sec";
								else
									span = (spanInMillis/60000)+"min"+(spanInMillis%60000/1000)+"sec";
								statusText.setText("Tracking "+getTarget().getName()+" for "+span);
								latitudeText.setText(neaten(Math.abs(point.getLatitude()))+((point.getLatitude()>=0)?"N":"S"));
								longitudeText.setText(neaten(Math.abs(point.getLongitude()))+((point.getLongitude()>=0)?"E":"W"));
								azimuthText.setText(neaten(point.getAzimuth()));
								elevationText.setText(neaten(point.getElevation()));
								altitudeText.setText(neaten(point.getAltitudeInKilometer()));
								slantRangeText.setText(String.valueOf((int)(point.getSlantRangeInKilometer())));
								velocitySecText.setText(neaten(point.getVelocity()));
								velocityHourText.setText(String.valueOf((int)(point.getVelocity()*60.0*60.0)));
							}
						});
					}
					if(((OperandSequence)currentOperand).hasNext()){
			        	openBluetoothSocketIfClosed();
			    		thread = new ReadWriteThread(getBluetoothSocket(),offsets,currentOperand,new MyListener());
			    		thread.start();
						return;
					}else if(currentOperand instanceof OperandTrack){
						Log.d(TAG,"%%%%%%ReTrack");
						((OperandTrack)currentOperand).track(rotateStatus,getSatCourse(),offsets);
			        	openBluetoothSocketIfClosed();
			    		thread = new ReadWriteThread(getBluetoothSocket(),offsets,currentOperand,new MyListener());
			    		thread.start();
			    		return;
					}else{
						currentOperand = null;
					}
				}else{
					currentOperand = null;
				}
			}
			handler.post(new Runnable() {
				@Override
				public void run() {
					writeIfPossible();
				}
			});
		}
	}
	private void autoSwitch(){
		Log.d(TAG,"Swithing:rotateStatus.getElevation()="+rotateStatus.getElevation()+",switchStatus="+switchStatus);
		if(!switchStatus.isAuto())return;
		OperandSwitch operand = null;
		if(rotateStatus.getElevation()>=0){
			if(switchStatus.getBlueMode()==SwitchStatus.LED_MODE_OFF||switchStatus.getRedMode()==SwitchStatus.LED_MODE_ON){
				Log.d(TAG,"Swithing:BLUE ON");
				operand = new OperandSwitch(switchStatus, SwitchStatus.LED_MODE_ON, switchStatus.getLaserMode(), SwitchStatus.LED_MODE_OFF);
			}
		}else{
			if(switchStatus.getBlueMode()==SwitchStatus.LED_MODE_ON||switchStatus.getRedMode()==SwitchStatus.LED_MODE_OFF){
				Log.d(TAG,"Swithing:RED ON");
				operand = new OperandSwitch(switchStatus, SwitchStatus.LED_MODE_OFF, switchStatus.getLaserMode(), SwitchStatus.LED_MODE_ON);
			}
		}
		writeNow(operand);
	}
    
	private long trackStartTime = -1;
    private void trackIfPossible(){
    	Log.d(TAG,"%%%trackIfPossible START");
		SatCourse course = getSatCourse();
		if(course==null){
	    	Log.d(TAG,"%%%trackIfPossible course is null");
			return;
		}
    	Log.d(TAG,"%%%trackIfPossible isBluetoothInactivated="+isBluetoothInactivated()+",getBluetoothDevice()="+getBluetoothDevice());
		if(!isBluetoothInactivated()&&getBluetoothDevice()==null||!offsets.isAzimuthCorrected()||!offsets.isTimeCorrected())return;
		speak("Start tracking");
		debugTextSatOrientation.setText(course.getPointList().size()+" lines acquired");
    	mainBackground.setBackgroundResource(R.drawable.laputa_ready);
    	trackStartTime = getCurrentTimeMillis();
		if(!operandTrack.track(rotateStatus,course,offsets))return;
    	Log.d(TAG,"%%%trackIfPossible writeIfPossible");
		writeIfPossible();
    	Log.d(TAG,"%%%trackIfPossible END");
	}
	
	//-------------------
	//	メニュー関連
	//-------------------
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu){
		super.onCreateOptionsMenu(menu);
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.main, menu);
		return true;
	}
	@Override
	public boolean onOptionsItemSelected(MenuItem item){
		Intent intent;
		switch(item.getItemId()){
			case R.id.menuSelectTarget:
				final List<Satellite> satList = getSatList();
				Log.d(TAG,"onOptionsItemSelected:satList.size()="+satList.size());
				speak("Entering target select mode");
				abortAllOperands();
				closeBluetoothSocket(true);
				intent = new Intent(this, TargetListActivity.class);
				intent.putExtra(TargetListActivity.EXTRA_NAME_SAT_LIST, new SatListContainer(satList));
				Log.d(TAG,"startActivity3");
				startActivity(intent);
				return true;
			case R.id.menuCalibrate:
				speak("Entering calibrate mode");
				abortAllOperands();
				closeBluetoothSocket(true);
				intent = new Intent(this, CalibrateActivity.class);
				intent.putExtra(EXTRA_NAME_BT_DEVICE, getBluetoothDevice());
				Log.d(TAG,"startActivity3");
				startActivity(intent);
				return true;
		}
		return false;
	}
	
	//---------------------
	//	取得したデータ
	//---------------------
	
    private void setTarget(Satellite target){
    	Log.d(TAG,"%%%setTarget START");
    	String msg = (target==null)?"Cannot select target":target.getName()+" is selected";
		speak(msg);
    	statusText.setText(msg);
    	trackIfPossible();
    	Log.d(TAG,"%%%setTarget END");
    }
	private Satellite target = null;
    private static final String ATTR_TARGET_CATNUM		 = "catnum";
    private static final Long DEFAULT_CATNUM = 25544l;	//ISS
	private Satellite getTarget(){
		if(target!=null)return target;
	    long targetCatnum = PreferenceManager.getDefaultSharedPreferences(this).getLong(ATTR_TARGET_CATNUM, 	Long.MAX_VALUE);
		target = findByCatnum(targetCatnum);
		if(target!=null)return target;
		target = findByCatnum(DEFAULT_CATNUM);
		if(target!=null)return target;
		List<Satellite> satList = getSatList();
		return (satList!=null&&satList.size()>0)?satList.get(0):null;
	}
	
	private List<Satellite> satList = null;
	private List<Satellite> getSatList(){
		if(satList!=null)return satList;
		try {
			satList = SatListParser.getInstance().getList();
			debugTextSatList.setText(satList.size()+" satellites acquired");
		} catch (Exception e) {
			DialogUtil.showSimpleDialog(this, "Connection Error", "Exception occured while acquiring the list of satellites.");
			satList = new ArrayList<Satellite>();
			e.printStackTrace();
		}
		return satList;
	}
	private Satellite findByCatnum(long catnum){
		List<Satellite> satList = getSatList();
		for(int i=0;i<satList.size();i++){
			if(satList.get(i).getCatnum()!=catnum)continue;
			return satList.get(i);
		}
		return null;
	}

	private GpsPositionInterface location = null;
	@Override
	public void onLocationChanged(Location location) {
		if(this.location!=null)return;
		this.location = new SimpleGpsPosition(location.getLongitude(), location.getLatitude(),location.getAltitude());
		lManager.removeUpdates(this);
		debugTextLocation.setText(neaten(location.getLongitude())+"/"+neaten(location.getLatitude())+"/"+neaten(location.getAltitude()));
    	trackIfPossible();
	}
	@Override
	public void onProviderDisabled(String provider) {}
	@Override
	public void onProviderEnabled(String provider) {}
	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {}

	private SatCourse _course = null;
	private long _courseEndTimeInMillis = -1;
	private SatCourse getSatCourse(){
		if(_course!=null&&_courseEndTimeInMillis-getCurrentTimeMillis()>10*60*1000)return _course;
		Satellite target = getTarget();
		if(target==null||location==null){
			Log.d(TAG,"getSatCourse:exit:target="+target+",location="+location);
			return _course;
		}
		try {
			_course = SatCourseWithAzElParser.getInstance().getCourse(target.getCatnum(), location.getLatitude(), location.getLongitude(), location.getHeightInMeter());
			_courseEndTimeInMillis = _course.getLastPoint().getTime().getTime();
			Log.d(TAG,"getSatCourse:_courseEndTimeInMillis="+_courseEndTimeInMillis+",now="+getCurrentTimeMillis()+",diff="+(_courseEndTimeInMillis-getCurrentTimeMillis()));
		} catch (Exception e) {
			e.printStackTrace();
			debugTextSatOrientation.setText("NG");
		}
		return _course;
	}
	
	public static class Offsets {
		
		private long _timeOffset = 0;
		private boolean timeCorrected = false;
		private double _azimuthOffset = 0.0;
		private boolean azimuthCorrected = false;
		
		public void setTimeOffset(long timeOffset){
			this._timeOffset = timeOffset;
			this.timeCorrected = true;
		}
		public boolean isTimeCorrected(){	return timeCorrected;	}
		public long getTimeOffset(){	return _timeOffset;	}

		public boolean setAzimuthOffset(double _azimuthOffset){
			if(Math.abs(_azimuthOffset-this._azimuthOffset)<10.0)return false;	//一定角度以上回さないと反応しない
			Log.d(TAG,"Offsets:setAzimuthOffset:"+this._azimuthOffset+"--->"+_azimuthOffset);
			this._azimuthOffset = _azimuthOffset;
			if(!this.azimuthCorrected)
				this.azimuthCorrected = true;
			return true;
		}
		public boolean isAzimuthCorrected(){	return azimuthCorrected;	}
		public double getAzimuthOffset(){	return _azimuthOffset;	}
	}
    private void setTimeOffset(){
    	offsets.setTimeOffset(TimeParser.getInstance().getTimeDiff());
    	debugTextTime.setText(offsets.getTimeOffset()+"ms");
    	trackIfPossible();
    }
    public long getCurrentTimeMillis(){
    	return System.currentTimeMillis() + offsets.getTimeOffset();
    }
	@Override
	public void onSensorChanged(SensorEvent event) {
		boolean isAzimuthCorrected = offsets.isAzimuthCorrected();
		if(!offsets.setAzimuthOffset(event.values[0]))return;
		debugTextCompass.setText(neaten(event.values[0]));
		if(!isAzimuthCorrected&&offsets.isAzimuthCorrected()){
			trackIfPossible();
		}else{
			//カタつくので補正は次の実行時にまかせる
			//writeNow(new OperandSwitch(switchStatus, SwitchStatus.LED_MODE_OFF, SwitchStatus.LASER_MODE_OFF, SwitchStatus.LED_MODE_OFF));
			//writeNow(new OperandRotate(rotateStatus, offsets, rotateStatus.getAzimuth(), rotateStatus.getElevation(), false));
		}
	}
	@Override
	public void onAccuracyChanged(Sensor sensor, int accuracy) {}
    
	private void speak(String msg){
		if(!isTextToSpeechActivated()||msg==null||msg.length()<=0)return;
		if(tts.isSpeaking())tts.stop();
		tts.speak(msg, TextToSpeech.QUEUE_FLUSH, null);
	}
	@Override
	public void onInit(int status) {
    	Log.d(TAG,"%%%onInit START");
		if(status == TextToSpeech.SUCCESS) {
			Locale locale = Locale.US;
			if(tts.isLanguageAvailable(locale) >= TextToSpeech.LANG_AVAILABLE) {
				tts.setLanguage(locale);
				debugTextTS.setText("OK");
			}else {
				debugTextTS.setText("Not supported locale");
			}
		}else{
			debugTextTS.setText("Init error.");
		}
    	Log.d(TAG,"%%%onInit END");
	}
	private static String neaten(double value){
		return String.valueOf(((int)(value*1000))/1000.0);
	}
	
	@Override
	public boolean onTouch(View v, MotionEvent event) {
		if(v.getId()!=laserActivateButton.getId())return false;
		if(event.getAction()==MotionEvent.ACTION_DOWN&&laserToggle.isChecked()){
	    	Log.d(TAG,"%%%onTouch:LASER ON BY ACTION_DOWN");
			writeNow(new OperandSwitch(switchStatus, switchStatus.getBlueMode(), SwitchStatus.LASER_MODE_ON, switchStatus.getRedMode()));
		}else if(event.getAction()==MotionEvent.ACTION_UP){
	    	Log.d(TAG,"%%%onTouch:LASER OFF BY ACTION_UP");
			writeNow(new OperandSwitch(switchStatus, switchStatus.getBlueMode(), SwitchStatus.LASER_MODE_OFF, switchStatus.getRedMode()));
		}
		return false;
	}
}