package st.tori.jpredict.levistone.activity;

import java.util.ArrayList;
import java.util.List;

import st.tori.jpredict.levistone.model.SatListContainer;
import st.tori.jpredict.levistone.model.Satellite;
import st.tori.jpredict.levistone.model.Target;
import st.tori.jpredict.levistone.model.Target.SimpleTarget;
import android.app.ListActivity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Parcelable;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;

public class TargetListActivity extends ListActivity {

	private static final String TAG = "TargetListActivity";
	
	public static final String EXTRA_NAME_SAT_LIST = "satList";
	public static final String EXTRA_NAME_TARGET_SAT = "targetSat";

	private SatListContainer container;
	
	private Target parentTarget = null;
	private static final int ID_SATELLITES	 = 1;
	private static final int ID_SPACESHIPS	 = 2;
	private static final int ID_GALAXIES	 = 3;
	private static final int ID_FIXED_STARS	 = 4;
	private static final int ID_PLANETS		 = 5;
	private Target.SimpleTarget targetSatellites = new Target.SimpleTarget(ID_SATELLITES,"Satellites");
	private Target.SimpleTarget targetSpaceships = new Target.SimpleTarget(ID_SPACESHIPS,"Spaceships");
	private Target.SimpleTarget targetGalaxies = new Target.SimpleTarget(ID_GALAXIES,"Galaxies");
	private Target.SimpleTarget targetFixedStars = new Target.SimpleTarget(ID_FIXED_STARS,"Fixed Stars");
	private Target.SimpleTarget targetPlanets = new Target.SimpleTarget(ID_PLANETS,"Planets");
	private Target.SimpleTarget targetNagiharae = new Target.SimpleTarget(ID_PLANETS,"NAGIHARAE");
	private Target[] rootTargetArray = new Target[]{targetSatellites,targetSpaceships,targetGalaxies,targetFixedStars,targetPlanets,targetNagiharae};

	private static String[] toStringArray(Target[] targetArray){
		if(targetArray==null||targetArray.length<=0)return new String[]{"NODATA"};
		String[] array = new String[targetArray.length];
		for(int i=0;i<targetArray.length;i++)
			array[i] = targetArray[i].getLabelOnTargetList();
		return array;
	}
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		container = getIntent().getParcelableExtra(EXTRA_NAME_SAT_LIST);
		if(container!=null)targetSatellites.setSubTargetArray(container.getSatList().toArray(new Satellite[container.getSatList().size()]));
		
		updateAdapter();
	}
	private void updateAdapter(){
		Target[] arrayTargets;
		if(parentTarget==null)
			arrayTargets = rootTargetArray;
		else
			arrayTargets = parentTarget.getSubTargetArray();
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, R.layout.target_list_row, toStringArray(arrayTargets));
		setListAdapter(adapter);
	}
	
    @Override
	protected void onListItemClick(ListView listView, View v, int position, long id) {
		super.onListItemClick(listView, v, position, id);
		Target selectedTarget = null;
		if(parentTarget==null&&position<rootTargetArray.length)
			selectedTarget = rootTargetArray[position];
		else if(parentTarget.getSubTargetArray()!=null&&position<parentTarget.getSubTargetArray().length)
			selectedTarget = parentTarget.getSubTargetArray()[position];
		Log.d(TAG,"Selected:selectedTarget="+selectedTarget);
		if(selectedTarget==null){
			return;
		}else if(selectedTarget instanceof Satellite){
			Intent intent = new Intent(this, MainActivity.class);
			intent.putExtra(EXTRA_NAME_TARGET_SAT, (Satellite)selectedTarget);
			startActivity(intent);
		}else if(selectedTarget.getSubTargetArray()!=null){
			parentTarget = selectedTarget;
			updateAdapter();
		}
	}

}
