package st.tori.jpredict.levistone.bizlogic;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import st.tori.jpredict.levistone.activity.MainActivity.Offsets;
import st.tori.jpredict.levistone.orm.Operand;
import android.bluetooth.BluetoothSocket;
import android.util.Log;

public class ReadWriteThread extends Thread {

	private static final String TAG = "ReadWriteThread";
	public static final String CMD_SUFFIX = "$";
	public static final String RET = "\n";
	
	private Offsets offsets;
	private Operand operand;
	private final ReadListener listener;
	private final InputStream mmInStream;
	private final OutputStream mmOutStream;

	public interface ReadListener{
		public void read(String cmdStr,String response);
	}
	public ReadWriteThread(BluetoothSocket socket,Offsets offsets,Operand operand,ReadListener listener) {
		this.offsets = offsets;
		this.operand = operand;
		this.listener = listener;
		if(socket==null){
			mmOutStream = null;
			mmInStream = null;
			return;
		}
		OutputStream tmpOut = null;
		InputStream tmpIn = null;
		try {
			tmpOut = socket.getOutputStream();
			tmpIn = socket.getInputStream();
		} catch (IOException e) {
			e.printStackTrace();
		}

		mmOutStream = tmpOut;
		mmInStream = tmpIn;
	}

	private long getCurrentTimeMillis(){
		return System.currentTimeMillis() + offsets.getTimeOffset();
	}
	public void run() {
		long waitUntilInMillis = operand.getWaitUntilInMillis();
		if(waitUntilInMillis>0){
			//Log.d(TAG,"now="+getCurrentTimeMillis()+"<--->waitUntilInMillis="+waitUntilInMillis);
			if(waitUntilInMillis>getCurrentTimeMillis()){
				try {
					Log.d(TAG,"wait for "+(waitUntilInMillis-getCurrentTimeMillis())+"ms");
					sleep(waitUntilInMillis-getCurrentTimeMillis());
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		}
		try {
			String write = operand.getCmdStr(offsets);
			operand.execting();
			if(mmOutStream!=null&&write!=null){
				Log.d(TAG,"Send:"+write);
				mmOutStream.write(write.getBytes());
			}
			if(listener!=null){
	    		String response;
				if(mmInStream==null){
					response = "DUMMY RESPONSE";
				}else{
		    		// Keep listening to the InputStream until an exception occurs
		    		byte[] buffer = new byte[1024]; // buffer store for the stream
		    		int bytes; // bytes returned from read()
		    		String val;
		    		response = "";
		    		while (true) {
	    				// Read from the InputStream
	    				bytes = mmInStream.read(buffer);
	    				if(bytes>=0){
		    				// Send the obtained bytes to the UI Activity
	    		    		val = new String(buffer,0,bytes);
	    		    		response += val;
	    					//Log.d(TAG,"Received1:"+val+",response="+response);
		    				if(val.endsWith(CMD_SUFFIX))break;
	    				}
		    		}
				}
				Log.d(TAG,"Received:"+response);
				listener.read(write,response);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
