package st.tori.jpredict.levistone.db;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

public class LevistoneData extends SQLiteOpenHelper {

	private static final String TAG = "LevistoneData";
	
	private static final String DATABASE_NAME = "levistone.db";
	private static final String TABLE_NAME = "calibrate";
	private static final String COLUMN_NAME_y_a_0	 = "y_a_0";
	private static final String COLUMN_NAME_y_a_360	 = "y_a_360";
	private static final String COLUMN_NAME_y_e_0	 = "y_e_0";
	private static final String COLUMN_NAME_y_e_90	 = "y_e_90";
	private static final String[] COLUMN_ARRAY = new String[]{COLUMN_NAME_y_a_0,COLUMN_NAME_y_a_360,COLUMN_NAME_y_e_0,COLUMN_NAME_y_e_90,};

	private static final int DATABASE_VERSION = 1;
	
	public LevistoneData(Context ctx){
		super(ctx,DATABASE_NAME,null,DATABASE_VERSION);
	}
	@Override
	public void onCreate(SQLiteDatabase db) {
		db.execSQL("CREATE TABLE "+TABLE_NAME+" ("+COLUMN_NAME_y_a_0+" INTEGER,"+COLUMN_NAME_y_a_360+" INTEGER,"+COLUMN_NAME_y_e_0+" INTEGER,"+COLUMN_NAME_y_e_90+" INTEGER)");
	}
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		db.execSQL("DROP TABLE "+TABLE_NAME);
		onCreate(db);
	}
	
	public static class YValues {
		private int y_a_0;
		private int y_a_360;
		private int y_e_0;
		private int y_e_90;
		public YValues(int y_a_0,int y_a_360,int y_e_0,int y_e_90){
			this.y_a_0 = y_a_0;
			this.y_a_360 = y_a_360;
			this.y_e_0 = y_e_0;
			this.y_e_90 = y_e_90;
		}
		public int getYA0(){	return y_a_0;	}
		public int getYA360(){	return y_a_360;	}
		public int getYE0(){	return y_e_0;	}
		public int getYE90(){	return y_e_90;	}
		public boolean equals(YValues obj){
			if(obj==null)return false;
			return (y_a_0==obj.y_a_0&&y_a_360==obj.y_a_360&&y_e_0==obj.y_e_0&&y_e_90==obj.y_e_90);
		}
	}

	public YValues getYValues(){
		SQLiteDatabase db = getReadableDatabase();
		try{
			return getYValues(db);
		}finally{
			db.close();
		}
	}
	private YValues getYValues(SQLiteDatabase db){
		YValues obj = null;
		Cursor cursor = null;
		try{
			cursor = db.query(TABLE_NAME, COLUMN_ARRAY, null, null, null, null, null);
			if(!cursor.moveToNext())return null;
			obj = new YValues(cursor.getInt(0), cursor.getInt(1), cursor.getInt(2), cursor.getInt(3));
			return obj;
		}finally{
			if(cursor!=null)cursor.close();
		}
	}
	public void saveYValues(YValues obj){
		SQLiteDatabase db = getWritableDatabase();
		try{
			YValues exist = getYValues(db);
			if(exist!=null){
				if(exist.equals(obj))return;
				deleteYValues(db);
			}
			insertYValues(db,obj);
		}finally{
			db.close();
		}
	}
	private void insertYValues(SQLiteDatabase db, YValues obj){
		ContentValues values = new ContentValues();
		values.put(COLUMN_NAME_y_a_0, obj.getYA0());
		values.put(COLUMN_NAME_y_a_360, obj.getYA360());
		values.put(COLUMN_NAME_y_e_0, obj.getYE0());
		values.put(COLUMN_NAME_y_e_90, obj.getYE90());
		db.insert(TABLE_NAME, null, values);
	}
	private void deleteYValues(SQLiteDatabase db){
		db.delete(TABLE_NAME, null, null);
	}

}
