package st.tori.jpredict.levistone.network.tsv;

import java.text.SimpleDateFormat;
import java.util.Locale;

import android.util.Log;

import st.tori.jpredict.levistone.AppLocal;
import st.tori.jpredict.levistone.model.course.SatCourse;
import st.tori.jpredict.levistone.model.course.SatCoursePointWithAzEl;

public class SatCourseWithAzElParser extends AbstractTsvFetcher {
	
	private static final String TAG = "SatCourseWithAzElParser";
	
	private  static SatCourseWithAzElParser instance;
	public static SatCourseWithAzElParser getInstance() {
		if (null==instance) instance = new SatCourseWithAzElParser();
		return instance;
	}
    private static String formatStr = "yyyy-MM-dd HH:mm:ss Z";
    private static SimpleDateFormat format = new SimpleDateFormat(formatStr, Locale.US);

	private String getUrl(long catnum,double latitude,double longitude,double heightInMeter) {
		return AppLocal.API_DOMAIN+"/api/sat/orientation.tsv?catnum="+catnum+"&latitude="+latitude+"&longitude="+longitude+"&heightInMeter="+heightInMeter+"&complete=true";
	}
	public SatCourse getCourse(long catnum,double latitude,double longitude,double heightInMeter) throws Exception {
		String urlStr = getUrl(catnum,latitude,longitude,heightInMeter);
		TsvReader reader = new TsvReader(urlStr);
		SatCourse course = new SatCourse();
		reader.read();
		while (reader.hasNext()) {
			TsvLine line = (TsvLine)reader.next();
			Log.d(TAG,"getCourse:line="+line.getRawString());
			if ("POS".equals(line.getType())) {
				SatCoursePointWithAzEl point = new SatCoursePointWithAzEl();
				point.setTime(format.parse(line.getStringAt(0).replace(" UTC", " GMT")));
				point.setLongitude(line.getDoubleAt(1));
				point.setLatitude(line.getDoubleAt(2));
				point.setHeading(line.getDoubleAt(3));
				point.setAltitudeInKilometer(line.getDoubleAt(4));
				point.setVelocity(line.getDoubleAt(5));
				
				point.setAzimuth(line.getDoubleAt(6));
				point.setElevation(line.getDoubleAt(7));
				point.setSlantRangeInKilometer(line.getDoubleAt(8));
				point.setSunlightVisibility(line.getIntAt(9));
				point.setInclination(line.getDoubleAt(10));
				
				course.addPoint(point);
			} else if ("BS".equals(line.getType())) {
				//Basic node
			}
			
		}
		return course;
	}
}