package st.tori.jpredict.levistone.orm;

import st.tori.jpredict.levistone.activity.MainActivity.Offsets;
import st.tori.jpredict.levistone.bizlogic.ReadWriteThread;
import st.tori.jpredict.levistone.util.StringUtil;
import android.util.Log;

public class OperandRotate extends Operand {

	private static final String TAG = "OperandRotate";
	
	private double azimuth;
	private double elevation;
	private boolean elSignum = true;
	private boolean raw = false;
	
	private double a_az = Double.MAX_VALUE;
	private double b_az = Double.MAX_VALUE;
	private double a_el = Double.MAX_VALUE;
	private double b_el = Double.MAX_VALUE;
	
	private RotateStatus status;
	private Offsets offsets;
	public OperandRotate(RotateStatus status,Offsets offsets){
		this.status = status;
		this.offsets = offsets;
	}
	public OperandRotate(RotateStatus status,Offsets offsets,double azimuth,double elevation,boolean raw){
		this.status = status;
		this.offsets = offsets;
		moveTo(azimuth,elevation,raw);
	}
	public OperandRotate(OperandRotate operandRotate) {
		this.status = operandRotate.status;
		this.offsets = operandRotate.offsets;
		this.a_az = operandRotate.a_az;
		this.b_az = operandRotate.b_az;
		this.a_el = operandRotate.a_el;
		this.b_el = operandRotate.b_el;
	}
	public void set(double a_az,double b_az,double a_el,double b_el){
		this.a_az = a_az;
		this.b_az = b_az;
		this.a_el = a_el;
		this.b_el = b_el;
	}
	public void moveTo(double azimuth,double elevation,boolean raw){
		this.azimuth = azimuth;
		this.elevation = Math.abs(elevation);
		this.elSignum = (elevation>=0.0);
		this.raw = raw;
		waiting();
	}
	private static double correctVirtualAzimuth(double vAzimuth){
		while(vAzimuth<0)vAzimuth += 360;
		while(vAzimuth>360)vAzimuth -= 360;
		return vAzimuth;
	}
	private static double correctVirtualElevation(double vElevation){
		if(vElevation<0)vElevation = 0;
		if(vElevation>90)vElevation = 90;
		return vElevation;
	}
	
	@Override
	public String getCmdStr(Offsets offsets) {
		if(raw){
			Log.d(TAG,"OperandRotate.getCmdStr1:azimuth(r)="+azimuth+",elevation(r)="+elevation);
			if(a_az!=Double.MAX_VALUE&&b_az!=Double.MAX_VALUE&&a_el!=Double.MAX_VALUE&&b_el!=Double.MAX_VALUE)
				Log.d(TAG,"OperandRotate.getCmdStr:azimuth(v)="+(azimuth-b_az)/a_az+",elevation(v)="+(elevation-b_el)/a_el+",elSignum="+elSignum);
		}else{
			Log.d(TAG,"OperandRotate.getCmdStr2:azimuth(v)="+azimuth+",elevation(v)="+elevation);
			if(a_az!=Double.MAX_VALUE&&b_az!=Double.MAX_VALUE&&a_el!=Double.MAX_VALUE&&b_el!=Double.MAX_VALUE)
				Log.d(TAG,"OperandRotate.getCmdStr2:azimuth(r)="+(a_az*azimuth+b_az)+",elevation(r)="+(a_el*elevation+b_el)+",elSignum="+elSignum);
		}
		double azimuthOffset = offsets.getAzimuthOffset();
		Log.d(TAG,"OperandRotate.getCmdStr:azimuthOffset="+azimuthOffset);
		double varAz = (!raw)?azimuth:(azimuth-b_az)/a_az;
		double varEl = (!raw)?elevation:(elevation-b_el)/a_el;
		varAz = correctVirtualAzimuth(varAz-azimuthOffset);
		varEl = correctVirtualElevation(varEl);
		status.setAzEl(varAz,(elSignum)?varEl:-varEl);
		if(!raw&&a_az!=Double.MAX_VALUE&&b_az!=Double.MAX_VALUE&&a_el!=Double.MAX_VALUE&&b_el!=Double.MAX_VALUE){
			this.azimuth = a_az * varAz + b_az;
			this.elevation = a_el * varEl + b_el;
			this.raw = true;
		}else if(!raw){
			this.azimuth = varAz;
			this.elevation = varEl;
		}else if(a_az!=Double.MAX_VALUE){
			this.azimuth = azimuth - a_az * azimuthOffset;
		}
		if(raw){
			Log.d(TAG,"OperandRotate.getCmdStr3:azimuth(r)="+azimuth+",elevation(r)="+elevation);
			if(a_az!=Double.MAX_VALUE&&b_az!=Double.MAX_VALUE&&a_el!=Double.MAX_VALUE&&b_el!=Double.MAX_VALUE)
				Log.d(TAG,"OperandRotate.getCmdStr3:azimuth(v)="+(azimuth-b_az)/a_az+",elevation(v)="+(elevation-b_el)/a_el+",elSignum="+elSignum);
			return CMD_PREFIX_OPERAND_ROTATE_RAW+StringUtil.expandInt((int)azimuth, 3)+StringUtil.expandInt((int)elevation, 3)+ReadWriteThread.CMD_SUFFIX;
		}else{
			Log.d(TAG,"OperandRotate.getCmdStr4:azimuth(v)="+azimuth+",elevation(v)="+elevation);
			if(a_az!=Double.MAX_VALUE&&b_az!=Double.MAX_VALUE&&a_el!=Double.MAX_VALUE&&b_el!=Double.MAX_VALUE)
				Log.d(TAG,"OperandRotate.getCmdStr4:azimuth(r)="+(a_az*azimuth+b_az)+",elevation(r)="+(a_el*elevation+b_el)+",elSignum="+elSignum);
			return CMD_PREFIX_OPERAND_ROTATE+StringUtil.expandInt((int)azimuth, 3)+StringUtil.expandInt((int)elevation, 2)+ReadWriteThread.CMD_SUFFIX;
		}
	}

	public double getAzimuth() {
		return azimuth;
	}
	public double getElevation() {
		return elevation;
	}
	public boolean getRaw() {
		return raw;
	}

}
