package st.tori.jpredict.levistone.orm;

import java.util.ArrayList;
import java.util.List;

import st.tori.jpredict.core.util.DateUtil;
import st.tori.jpredict.levistone.activity.MainActivity.Offsets;
import st.tori.jpredict.levistone.model.course.SatCourse;
import st.tori.jpredict.levistone.model.course.SatCoursePointWithAzEl;
import android.util.Log;

public class OperandTrack extends OperandSequence {

	private static final String TAG = "OperandTrack";
	
	private OperandRotate operandRotate;
	private OperandSwitch operandSwitch;
	public OperandTrack(OperandRotate operandRotate,OperandSwitch operandSwitch){
		this.operandRotate = operandRotate;
		this.operandSwitch = operandSwitch;
	}
	public boolean track(RotateStatus rotateStatus,SatCourse course,Offsets offsets){
		List<SatCoursePointWithAzEl> pointList = course.getPointList();
		List<Operand> _operandList = new ArrayList<Operand>();
		List<SatCoursePointWithAzEl> _pointList = new ArrayList<SatCoursePointWithAzEl>();
		SatCoursePointWithAzEl lastPoint = null;
		long since = System.currentTimeMillis()+offsets.getTimeOffset();
		long fromTime,toTime;
		long until = since + 3*60*1000;	//1分間
		Log.d(TAG,"track:since="+since+",until="+until);
		double inclination = Double.MAX_VALUE;
		for(int i=0;i<pointList.size();i++){
			SatCoursePointWithAzEl point = pointList.get(i);
			if(inclination==Double.MAX_VALUE)inclination = point.getInclination();
			toTime = point.getTime().getTime();
			if(lastPoint!=null){
				Log.d(TAG,"track:lstPoint.getTime()="+DateUtil.getDateStr(lastPoint.getTime()));
				Log.d(TAG,"track:curPoint.getTime()="+DateUtil.getDateStr(point.getTime())+",toTime["+i+"]="+toTime);
			}
			if(lastPoint!=null&&since<toTime&&toTime<until){
				fromTime = Math.max(since, lastPoint.getTime().getTime());
				toTime = Math.min(until, toTime);
				Log.d(TAG,"track:fromTime="+fromTime+",toTime="+toTime);
				long step = getProperStepInMillis(lastPoint.getSlantRangeInKilometer());
				for(long time=fromTime;time<toTime;time+=step){
					SatCoursePointWithAzEl mean = new SatCoursePointWithAzEl(lastPoint,point,time);
					OperandRotate operand = new OperandRotate(operandRotate);
					operand.setWaitUntilInMillis(time);
					operand.moveTo(mean.getAzimuth(), mean.getElevation(), false);
					_operandList.add(operand);
					_pointList.add(mean);
					step = getProperStepInMillis(mean.getSlantRangeInKilometer());
				}
			}else if(until<toTime){
				break;
			}
			lastPoint = point;
		}
		rotateStatus.setInclination(inclination);
		Log.d(TAG,"operandList.size()="+_operandList.size());
		if(_operandList.size()<=0)return false;
		setOperands(_operandList,_pointList);
		waiting();
		return true;
	}
	private static long getProperStepInMillis(double slantRangeInKilometer){
		if(slantRangeInKilometer>2000){
			return 15000;	//15秒ごと
		}else if(slantRangeInKilometer>1000){
			return 5000;	//5秒ごと
		}else{
			return 3000;	//3秒ごと
		}
	}
	
	public void setOperands(List<Operand> operandList,List<SatCoursePointWithAzEl> pointList){
		this.pointList = pointList;
		super.setOperands(operandList);
	}
	private List<SatCoursePointWithAzEl> pointList;
	public SatCoursePointWithAzEl getSatCoursePointWithAzEl(){
		return pointList.get(index);
	}

}
